@@ -29,8 +29,8 @@
  * given pad can handle. They are also stored in the registry along with
  * a description of the element.
  *
- * Caps are exposed on the element pads using the gst_pad_get_caps() pad 
- * function. This function describes the possible types that the pad can 
+ * Caps are exposed on the element pads using the gst_pad_get_caps() pad
+ * function. This function describes the possible types that the pad can
  * handle or produce at runtime.
  *
  * Caps are also attached to buffers to describe to content of the data
@@ -47,8 +47,8 @@
  *       "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
  *       "framerate", G_TYPE_DOUBLE, 25.0,
  *       "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
- *       "width", G_TYPE_INT, 320, 
- *       "height", G_TYPE_INT, 240, 
+ *       "width", G_TYPE_INT, 320,
+ *       "height", G_TYPE_INT, 240,
  *       NULL);
  *  </programlisting>
  * </example>
@@ -91,6 +91,13 @@
 #define IS_WRITABLE(caps) \
   (g_atomic_int_get (&(caps)->refcount) == 1)
 
+#if GLIB_CHECK_VERSION (2, 10, 0)
+#define ALLOC_CAPS()    g_slice_new (GstCaps)
+#define FREE_CAPS(caps) g_slice_free (GstCaps, caps)
+#else
+#define ALLOC_CAPS()    g_new (GstCaps, 1)
+#define FREE_CAPS(caps) g_free (caps)
+#endif
 
 static void gst_caps_transform_to_string (const GValue * src_value,
     GValue * dest_value);
@@ -129,10 +136,11 @@ gst_caps_get_type (void)
 GstCaps *
 gst_caps_new_empty (void)
 {
-  GstCaps *caps = g_new0 (GstCaps, 1);
+  GstCaps *caps = ALLOC_CAPS ();
 
-  g_atomic_int_inc (&caps->refcount);
   caps->type = GST_TYPE_CAPS;
+  caps->refcount = 1;
+  caps->flags = 0;
   caps->structs = g_ptr_array_new ();
 
 #ifdef DEBUG_REFCOUNT
@@ -294,7 +302,7 @@ _gst_caps_free (GstCaps * caps)
 #ifdef USE_POISONING
   memset (caps, 0xff, sizeof (GstCaps));
 #endif
-  g_free (caps);
+  FREE_CAPS (caps);
 }
 
 /**
@@ -384,9 +392,8 @@ gst_caps_unref (GstCaps * caps)
   g_return_if_fail (GST_CAPS_REFCOUNT_VALUE (caps) > 0);
 
   /* if we ended up with the refcount at zero, free the caps */
-  if (g_atomic_int_dec_and_test (&caps->refcount)) {
+  if (G_UNLIKELY (g_atomic_int_dec_and_test (&caps->refcount)))
     _gst_caps_free (caps);
-  }
 }
 
 GType
@@ -414,43 +421,119 @@ gst_static_caps_get_type (void)
 GstCaps *
 gst_static_caps_get (GstStaticCaps * static_caps)
 {
-  GstCaps *caps = (GstCaps *) static_caps;
-  gboolean ret;
+  GstCaps *caps;
+
+  g_return_val_if_fail (static_caps != NULL, NULL);
+
+  caps = (GstCaps *) static_caps;
 
   if (caps->type == 0) {
-    if (static_caps->string == NULL) {
-      g_warning ("static caps is NULL");
-      return NULL;
-    }
+    if (G_UNLIKELY (static_caps->string == NULL))
+      goto no_string;
 
     caps->type = GST_TYPE_CAPS;
     /* initialize the caps to a refcount of 1 so the caps can be writable... */
     gst_atomic_int_set (&caps->refcount, 1);
     caps->structs = g_ptr_array_new ();
-    ret = gst_caps_from_string_inplace (caps, static_caps->string);
 
-    if (!ret) {
+    /* convert to string */
+    if (G_UNLIKELY (!gst_caps_from_string_inplace (caps, static_caps->string)))
       g_critical ("Could not convert static caps \"%s\"", static_caps->string);
-    }
   }
   /* ref the caps, makes it not writable */
   gst_caps_ref (caps);
 
   return caps;
+
+  /* ERRORS */
+no_string:
+  {
+    g_warning ("static caps string is NULL");
+    return NULL;
+  }
 }
 
 /* manipulation */
-
 static GstStructure *
 gst_caps_remove_and_get_structure (GstCaps * caps, guint idx)
 {
-  /* don't use index_fast, gst_caps_simplify relies on the order */
+  /* don't use index_fast, gst_caps_do_simplify relies on the order */
   GstStructure *s = g_ptr_array_remove_index (caps->structs, idx);
 
   gst_structure_set_parent_refcount (s, NULL);
   return s;
 }
 
+static gboolean
+gst_structure_is_equal_foreach (GQuark field_id, const GValue * val2,
+    gpointer data)
+{
+  GstStructure *struct1 = (GstStructure *) data;
+  const GValue *val1 = gst_structure_id_get_value (struct1, field_id);
+
+  if (val1 == NULL)
+    return FALSE;
+  if (gst_value_compare (val1, val2) == GST_VALUE_EQUAL) {
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+static gboolean
+gst_caps_structure_is_subset_field (GQuark field_id, const GValue * value,
+    gpointer user_data)
+{
+  GstStructure *subtract_from = user_data;
+  GValue subtraction = { 0, };
+  const GValue *other;
+  gint res;
+
+  other = gst_structure_id_get_value (subtract_from, field_id);
+  if (!other) {
+    /* field is missing in one set */
+    return FALSE;
+  }
+  /*
+   * [1,2] - 1 = 2
+   * 1 - [1,2] = ???
+   */
+  if (!gst_value_subtract (&subtraction, other, value)) {
+    /* empty result -> values are the same, or first was a value and
+     * second was a list
+     * verify that result is empty by swapping args */
+    if (!gst_value_subtract (&subtraction, value, other)) {
+      return TRUE;
+    }
+    g_value_unset (&subtraction);
+    return FALSE;
+  }
+
+  res = gst_value_compare (&subtraction, other);
+  g_value_unset (&subtraction);
+
+  if (res == GST_VALUE_EQUAL) {
+    /* value was empty ? */
+    return FALSE;
+  } else {
+    return TRUE;
+  }
+}
+
+static gboolean
+gst_caps_structure_is_subset (const GstStructure * minuend,
+    const GstStructure * subtrahend)
+{
+  if ((minuend->name != subtrahend->name) ||
+      (gst_structure_n_fields (minuend) !=
+          gst_structure_n_fields (subtrahend))) {
+    return FALSE;
+  }
+
+  return gst_structure_foreach ((GstStructure *) subtrahend,
+      gst_caps_structure_is_subset_field, (gpointer) minuend);
+}
+
 /**
  * gst_caps_append:
  * @caps1: the #GstCaps that will be appended to
@@ -492,6 +575,63 @@ gst_caps_append (GstCaps * caps1, GstCaps * caps2)
   gst_caps_unref (caps2);       /* guaranteed to free it */
 }
 
+/**
+ * gst_caps_merge:
+ * @caps1: the #GstCaps that will take the new entries
+ * @caps2: the #GstCaps to merge in
+ *
+ * Appends the structures contained in @caps2 to @caps1 if they are not yet
+ * expressed by @caps1. The structures in @caps2 are not copied -- they are
+ * transferred to @caps1, and then @caps2 is freed.
+ * If either caps is ANY, the resulting caps will be ANY.
+ *
+ * Since: 0.10.10
+ */
+void
+gst_caps_merge (GstCaps * caps1, GstCaps * caps2)
+{
+  GstStructure *structure;
+  int i;
+
+  g_return_if_fail (GST_IS_CAPS (caps1));
+  g_return_if_fail (GST_IS_CAPS (caps2));
+  g_return_if_fail (IS_WRITABLE (caps1));
+  g_return_if_fail (IS_WRITABLE (caps2));
+
+#ifdef USE_POISONING
+  CAPS_POISON (caps2);
+#endif
+  if (gst_caps_is_any (caps1)) {
+    for (i = caps2->structs->len - 1; i >= 0; i--) {
+      structure = gst_caps_remove_and_get_structure (caps2, i);
+      gst_structure_free (structure);
+    }
+  } else if (gst_caps_is_any (caps2)) {
+    caps1->flags |= GST_CAPS_FLAGS_ANY;
+    for (i = caps1->structs->len - 1; i >= 0; i--) {
+      structure = gst_caps_remove_and_get_structure (caps1, i);
+      gst_structure_free (structure);
+    }
+  } else {
+    int len = caps2->structs->len;
+
+    for (i = 0; i < len; i++) {
+      structure = gst_caps_remove_and_get_structure (caps2, 0);
+      gst_caps_merge_structure (caps1, structure);
+    }
+    /* this is too naive
+       GstCaps *com = gst_caps_intersect (caps1, caps2);
+       GstCaps *add = gst_caps_subtract (caps2, com);
+
+       GST_DEBUG ("common : %d", gst_caps_get_size (com));
+       GST_DEBUG ("adding : %d", gst_caps_get_size (add));
+       gst_caps_append (caps1, add);
+       gst_caps_unref (com);
+     */
+  }
+  gst_caps_unref (caps2);       /* guaranteed to free it */
+}
+
 /**
  * gst_caps_append_structure:
  * @caps: the #GstCaps that will be appended to
@@ -506,7 +646,7 @@ gst_caps_append_structure (GstCaps * caps, GstStructure * structure)
   g_return_if_fail (GST_IS_CAPS (caps));
   g_return_if_fail (IS_WRITABLE (caps));
 
-  if (structure) {
+  if (G_LIKELY (structure)) {
     g_return_if_fail (structure->parent_refcount == NULL);
 #if 0
 #ifdef USE_POISONING
@@ -518,7 +658,7 @@ gst_caps_append_structure (GstCaps * caps, GstStructure * structure)
   }
 }
 
-/*
+/**
  * gst_caps_remove_structure:
  * @caps: the #GstCaps to remove from
  * @idx: Index of the structure to remove
@@ -540,22 +680,49 @@ gst_caps_remove_structure (GstCaps * caps, guint idx)
 }
 
 /**
- * gst_caps_split_one:
- * @caps: a #GstCaps
+ * gst_caps_merge_structure:
+ * @caps: the #GstCaps that will the the new structure
+ * @structure: the #GstStructure to merge
  *
- * This function is not implemented.
- *
- * Returns: NULL
+ * Appends @structure to @caps if its not already expressed by @caps.  The
+ * structure is not copied; @caps becomes the owner of @structure.
  */
-GstCaps *
-gst_caps_split_one (GstCaps * caps)
+void
+gst_caps_merge_structure (GstCaps * caps, GstStructure * structure)
 {
-  /* FIXME */
-  g_critical ("unimplemented");
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
 
-  return NULL;
+  if (G_LIKELY (structure)) {
+    GstStructure *structure1;
+    int i;
+    gboolean unique = TRUE;
+
+    g_return_if_fail (structure->parent_refcount == NULL);
+#if 0
+#ifdef USE_POISONING
+    STRUCTURE_POISON (structure);
+#endif
+#endif
+    /* check each structure */
+    for (i = caps->structs->len - 1; i >= 0; i--) {
+      structure1 = gst_caps_get_structure (caps, i);
+      /* if structure is a subset of structure1, then skip it */
+      if (gst_caps_structure_is_subset (structure1, structure)) {
+        unique = FALSE;
+        break;
+      }
+    }
+    if (unique) {
+      gst_structure_set_parent_refcount (structure, &caps->refcount);
+      g_ptr_array_add (caps->structs, structure);
+    } else {
+      gst_structure_free (structure);
+    }
+  }
 }
 
+
 /**
  * gst_caps_get_size:
  * @caps: a #GstCaps
@@ -591,7 +758,6 @@ GstStructure *
 gst_caps_get_structure (const GstCaps * caps, guint index)
 {
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  g_return_val_if_fail (index >= 0, NULL);
   g_return_val_if_fail (index < caps->structs->len, NULL);
 
   return g_ptr_array_index (caps->structs, index);
@@ -767,22 +933,6 @@ gst_caps_is_fixed (const GstCaps * caps)
   return gst_structure_foreach (structure, gst_caps_is_fixed_foreach, NULL);
 }
 
-static gboolean
-gst_structure_is_equal_foreach (GQuark field_id, const GValue * val2,
-    gpointer data)
-{
-  GstStructure *struct1 = (GstStructure *) data;
-  const GValue *val1 = gst_structure_id_get_value (struct1, field_id);
-
-  if (val1 == NULL)
-    return FALSE;
-  if (gst_value_compare (val1, val2) == GST_VALUE_EQUAL) {
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 /**
  * gst_caps_is_equal_fixed:
  * @caps1: the #GstCaps to test
@@ -875,9 +1025,9 @@ gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
  * <note>This function does not work reliably if optional properties for caps
  * are included on one caps and omitted on the other.</note>
  *
- * This function deals correctly with passing NULL for any of the caps. 
+ * This function deals correctly with passing NULL for any of the caps.
  *
- * Returns: TRUE if both caps are equal. 
+ * Returns: TRUE if both caps are equal.
  */
 gboolean
 gst_caps_is_equal (const GstCaps * caps1, const GstCaps * caps2)
@@ -1309,7 +1459,7 @@ gst_caps_compare_structures (gconstpointer one, gconstpointer two)
   const GstStructure *struct1 = *((const GstStructure **) one);
   const GstStructure *struct2 = *((const GstStructure **) two);
 
-  /* FIXME: this orders aphabetically, but ordering the quarks might be faster
+  /* FIXME: this orders alphabetically, but ordering the quarks might be faster
      So what's the best way? */
   ret = strcmp (gst_structure_get_name (struct1),
       gst_structure_get_name (struct2));
@@ -1319,30 +1469,6 @@ gst_caps_compare_structures (gconstpointer one, gconstpointer two)
   return gst_structure_n_fields (struct2) - gst_structure_n_fields (struct1);
 }
 
-/**
- * gst_caps_simplify:
- * @caps: a #GstCaps to simplify
- *
- * Creates a new #GstCaps that represents the same set of formats as
- * @caps, but simpler.  Component structures that are identical are
- * merged.  Component structures that have ranges or lists that can
- * be merged are also merged.
- *
- * Returns: the new #GstCaps
- */
-GstCaps *
-gst_caps_simplify (const GstCaps * caps)
-{
-  GstCaps *ret;
-
-  g_return_val_if_fail (caps != NULL, NULL);
-
-  ret = gst_caps_copy (caps);
-  gst_caps_do_simplify (ret);
-
-  return ret;
-}
-
 typedef struct
 {
   GQuark name;
@@ -1560,26 +1686,28 @@ gst_caps_load_thyself (xmlNodePtr parent)
  * Replaces *caps with @newcaps.  Unrefs the #GstCaps in the location
  * pointed to by @caps, if applicable, then modifies @caps to point to
  * @newcaps. An additional ref on @newcaps is taken.
+ *
+ * This function does not take any locks so you might want to lock
+ * the object owning @caps pointer.
  */
 void
 gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
 {
   GstCaps *oldcaps;
 
-#if 0                           /* disable this, since too many plugins rely on undefined behavior */
-#ifdef USE_POISONING
-  //if (newcaps) CAPS_POISON (newcaps);
-#endif
-#endif
+  g_return_if_fail (caps != NULL);
+
   oldcaps = *caps;
 
-  if (newcaps)
-    gst_caps_ref (newcaps);
+  if (newcaps != oldcaps) {
+    if (newcaps)
+      gst_caps_ref (newcaps);
 
-  *caps = newcaps;
+    *caps = newcaps;
 
-  if (oldcaps)
-    gst_caps_unref (oldcaps);
+    if (oldcaps)
+      gst_caps_unref (oldcaps);
+  }
 }
 
 /**
@@ -1589,6 +1717,12 @@ gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
  * Converts @caps to a string representation.  This string representation
  * can be converted back to a #GstCaps by gst_caps_from_string().
  *
+ * For debugging purposes its easier to do something like this:
+ * <programlisting>
+ *  GST_LOG ("caps are %" GST_PTR_FORMAT, caps);
+ * </programlisting>
+ * This prints the caps in human readble form.
+ *
  * Returns: a newly allocated string representing @caps.
  */
 gchar *
