@@ -675,6 +675,11 @@ no_direction:
  * Removes @pad from @element. @pad will be destroyed if it has not been
  * referenced elsewhere.
  *
+ * This function is used by plugin developers and should not be used
+ * by applications. Pads that were dynamically requested from elements
+ * with gst_element_get_request_pad() should be released with the
+ * gst_element_release_request_pad() function instead.
+ *
  * Returns: TRUE if the pad could be removed. Can return FALSE if the
  * pad is not belonging to the provided element.
  *
@@ -741,8 +746,7 @@ not_our_pad:
     /* FIXME, locking order? */
     GST_OBJECT_LOCK (element);
     g_critical ("Padname %s:%s does not belong to element %s when removing",
-        GST_ELEMENT_NAME (GST_PAD_PARENT (pad)), GST_PAD_NAME (pad),
-        GST_ELEMENT_NAME (element));
+        GST_DEBUG_PAD_NAME (pad), GST_ELEMENT_NAME (element));
     GST_OBJECT_UNLOCK (element);
     GST_OBJECT_UNLOCK (pad);
     return FALSE;
@@ -847,7 +851,8 @@ gst_element_request_pad (GstElement * element, GstPadTemplate * templ,
  * @name: the name of the request #GstPad to retrieve.
  *
  * Retrieves a pad from the element by name. This version only retrieves
- * request pads.
+ * request pads. The pad should be released with 
+ * gst_element_release_request_pad().
  *
  * Returns: requested #GstPad if found, otherwise NULL. Unref after usage.
  */
@@ -1745,7 +1750,7 @@ interrupted:
 /**
  * gst_element_get_state:
  * @element: a #GstElement to get the state of.
- * @state: a pointer to #GstState to hold the state. Can be NULL.
+ * @state: a pointer to #GstState to hold the state. Can be %NULL.
  * @pending: a pointer to #GstState to hold the pending state.
  *           Can be %NULL.
  * @timeout: a #GstClockTime to specify the timeout for an async
@@ -1859,9 +1864,9 @@ nothing_aborted:
  * pending state if any. This function is used   
  * by elements that do asynchronous state changes.       
  * The core will normally call this method automatically when an         
- * element returned SUCCESS from the state change function.      
- * Elements that return ASYNC from the change_state function should      
- * eventually call this method from the streaming thread to signal       
+ * element returned %GST_STATE_CHANGE_SUCCESS from the state change function.      
+ * Elements that return %GST_STATE_CHANGE_ASYNC from the change_state function
+ * should eventually call this method from the streaming thread to signal       
  * successfull state change completion.          
  *       
  * If after calling this method the element still has not reached        
@@ -1964,11 +1969,11 @@ complete:
  *
  * Brings the element to the lost state. The current state of the
  * element is copied to the pending state so that any call to
- * gst_element_get_state() will return ASYNC.
+ * gst_element_get_state() will return %GST_STATE_CHANGE_ASYNC.
  *
  * This is mostly used for elements that lost their preroll buffer
- * in the PAUSED state after a flush, they become PAUSED again
- * if a new preroll buffer is queued.
+ * in the %GST_STATE_PAUSED state after a flush, they become %GST_STATE_PAUSED
+ * again if a new preroll buffer is queued.
  * This function can only be called when the element is currently
  * not in error or an async state change.
  *
@@ -2246,88 +2251,126 @@ invalid_return:
   }
 }
 
-/* gst_iterator_fold functions for pads_activate */
-
+/* gst_iterator_fold functions for pads_activate
+ * Note how we don't stop the iterator when we fail an activation. This is
+ * probably a FIXME since when one pad activation fails, we don't want to
+ * continue our state change. */
 static gboolean
 activate_pads (GstPad * pad, GValue * ret, gboolean * active)
 {
   if (!gst_pad_set_active (pad, *active))
     g_value_set_boolean (ret, FALSE);
-  else if (!*active)
-    gst_pad_set_caps (pad, NULL);
 
+  /* unref the object that was reffed for us by _fold */
   gst_object_unref (pad);
   return TRUE;
 }
 
-/* returns false on error or early cutout of the fold, true otherwise */
+/* set the caps on the pad to NULL */
 static gboolean
-iterator_fold_with_resync (GstIterator * iter, GstIteratorFoldFunction func,
-    GValue * ret, gpointer user_data)
+clear_caps (GstPad * pad, GValue * ret, gboolean * active)
 {
-  GstIteratorResult ires;
-  gboolean res = TRUE;
-
-  while (1) {
-    ires = gst_iterator_fold (iter, func, ret, user_data);
-
-    switch (ires) {
-      case GST_ITERATOR_RESYNC:
-        gst_iterator_resync (iter);
-        break;
-      case GST_ITERATOR_DONE:
-        res = TRUE;
-        goto done;
-      default:
-        res = FALSE;
-        goto done;
-    }
-  }
-
-done:
-  return res;
+  gst_pad_set_caps (pad, NULL);
+  gst_object_unref (pad);
+  return TRUE;
 }
 
-/* is called with STATE_LOCK
- */
+/* returns false on error or early cutout (will never happen because the fold
+ * function always returns TRUE, see FIXME above) of the fold, true if all
+ * pads in @iter were (de)activated successfully. */
 static gboolean
-gst_element_pads_activate (GstElement * element, gboolean active)
+iterator_activate_fold_with_resync (GstIterator * iter,
+    GstIteratorFoldFunction func, gpointer user_data)
 {
-  GValue ret = { 0, };
-  GstIterator *iter;
-  gboolean fold_ok;
+  GstIteratorResult ires;
+  GValue ret = { 0 };
 
-  GST_DEBUG_OBJECT (element, "pads_activate with active %d", active);
   /* no need to unset this later, it's just a boolean */
   g_value_init (&ret, G_TYPE_BOOLEAN);
   g_value_set_boolean (&ret, TRUE);
 
-  if (active)
-    iter = gst_element_iterate_src_pads (element);
-  else
-    iter = gst_element_iterate_sink_pads (element);
-  fold_ok = iterator_fold_with_resync
-      (iter, (GstIteratorFoldFunction) activate_pads, &ret, &active);
-  gst_iterator_free (iter);
-  if (!fold_ok || !g_value_get_boolean (&ret)) {
-    GST_DEBUG_OBJECT (element, "pads_activate failed");
-    return FALSE;
+  while (1) {
+    ires = gst_iterator_fold (iter, func, &ret, user_data);
+    switch (ires) {
+      case GST_ITERATOR_RESYNC:
+        /* need to reset the result again */
+        g_value_set_boolean (&ret, TRUE);
+        gst_iterator_resync (iter);
+        break;
+      case GST_ITERATOR_DONE:
+        /* all pads iterated, return collected value */
+        goto done;
+      default:
+        /* iterator returned _ERROR or premature end with _OK, 
+         * mark an error and exit */
+        g_value_set_boolean (&ret, FALSE);
+        goto done;
+    }
   }
+done:
+  /* return collected value */
+  return g_value_get_boolean (&ret);
+}
 
-  if (active)
-    iter = gst_element_iterate_sink_pads (element);
-  else
-    iter = gst_element_iterate_src_pads (element);
-  fold_ok = iterator_fold_with_resync
-      (iter, (GstIteratorFoldFunction) activate_pads, &ret, &active);
+/* is called with STATE_LOCK
+ *
+ * Pads are activated from source pads to sinkpads. 
+ */
+static gboolean
+gst_element_pads_activate (GstElement * element, gboolean active)
+{
+  GstIterator *iter;
+  gboolean res;
+
+  GST_DEBUG_OBJECT (element, "pads_activate with active %d", active);
+
+  iter = gst_element_iterate_src_pads (element);
+  res =
+      iterator_activate_fold_with_resync (iter,
+      (GstIteratorFoldFunction) activate_pads, &active);
   gst_iterator_free (iter);
-  if (!fold_ok || !g_value_get_boolean (&ret)) {
-    GST_DEBUG_OBJECT (element, "pads_activate failed");
-    return FALSE;
+  if (G_UNLIKELY (!res))
+    goto src_failed;
+
+  iter = gst_element_iterate_sink_pads (element);
+  res =
+      iterator_activate_fold_with_resync (iter,
+      (GstIteratorFoldFunction) activate_pads, &active);
+  gst_iterator_free (iter);
+  if (G_UNLIKELY (!res))
+    goto sink_failed;
+
+  if (!active) {
+    /* clear the caps on all pads, this should never fail */
+    iter = gst_element_iterate_pads (element);
+    res =
+        iterator_activate_fold_with_resync (iter,
+        (GstIteratorFoldFunction) clear_caps, &active);
+    gst_iterator_free (iter);
+    if (G_UNLIKELY (!res))
+      goto caps_failed;
   }
 
   GST_DEBUG_OBJECT (element, "pads_activate successful");
+
   return TRUE;
+
+  /* ERRORS */
+src_failed:
+  {
+    GST_DEBUG_OBJECT (element, "source pads_activate failed");
+    return FALSE;
+  }
+sink_failed:
+  {
+    GST_DEBUG_OBJECT (element, "sink pads_activate failed");
+    return FALSE;
+  }
+caps_failed:
+  {
+    GST_DEBUG_OBJECT (element, "failed to clear caps on pads");
+    return FALSE;
+  }
 }
 
 /* is called with STATE_LOCK */
@@ -2643,8 +2686,8 @@ gst_element_set_bus_func (GstElement * element, GstBus * bus)
  * @element: a #GstElement to set the bus of.
  * @bus: the #GstBus to set.
  *
- * Sets the bus of the element.  For internal use only, unless you're
- * testing elements.
+ * Sets the bus of the element. Increases the refcount on the bus.
+ * For internal use only, unless you're testing elements.
  *
  * MT safe.
  */
@@ -2679,8 +2722,8 @@ gst_element_get_bus (GstElement * element)
   g_return_val_if_fail (GST_IS_ELEMENT (element), result);
 
   GST_OBJECT_LOCK (element);
-  result = GST_ELEMENT_BUS (element);
-  gst_object_ref (result);
+  if ((result = GST_ELEMENT_BUS (element)))
+    gst_object_ref (result);
   GST_OBJECT_UNLOCK (element);
 
   GST_DEBUG_OBJECT (element, "got bus %" GST_PTR_FORMAT, result);
