@@ -21,23 +21,54 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstelementfactory
+ * @short_description: Create GstElements from a factory
+ * @see_also: #GstElement, #GstPlugin, #GstPluginFeature, #GstPadTemplate.
+ *
+ * #GstElementFactory is used to create instances of elements. A
+ * GstElementfactory can be added to a #GstPlugin as it is also a
+ * #GstPluginFeature.
+ *
+ * Use the gst_element_factory_find() and gst_element_factory_create()
+ * functions to create element instances or use gst_element_factory_make() as a
+ * convenient shortcut.
+ *
+ * The following code example shows you how to create a GstFileSrc element.
+ *
+ * <example>
+ * <title>Using an element factory</title>
+ * <programlisting language="c">
+ *   #include &lt;gst/gst.h&gt;
+ *   GstElement *src;
+ *   GstElementFactory *srcfactory;
+ *   gst_init(&amp;argc,&amp;argv);
+ *   srcfactory = gst_element_factory_find("filesrc");
+ *   g_return_if_fail(srcfactory != NULL);
+ *   src = gst_element_factory_create(srcfactory,"src");
+ *   g_return_if_fail(src != NULL);
+ *   ...
+ * </programlisting>
+ * </example>
+ *
+ * Last reviewed on 2005-11-23 (0.9.5)
+ */
+
 #include "gst_private.h"
 
 #include "gstelement.h"
-#include "gstregistrypool.h"
 #include "gstinfo.h"
 #include "gsturi.h"
-#ifndef GST_DISABLE_REGISTRY
-#include "registries/gstxmlregistry.h"  /* g_critical in gst_element_factory_create */
-#endif
+#include "gstregistry.h"
 
 GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
 #define GST_CAT_DEFAULT element_factory_debug
 
 static void gst_element_factory_class_init (GstElementFactoryClass * klass);
 static void gst_element_factory_init (GstElementFactory * factory);
-
-static void gst_element_factory_unload_thyself (GstPluginFeature * feature);
+static void gst_element_factory_finalize (GObject * object);
+void __gst_element_details_clear (GstElementDetails * dp);
+static void gst_element_factory_cleanup (GstElementFactory * factory);
 
 static GstPluginFeatureClass *parent_class = NULL;
 
@@ -48,7 +79,7 @@ gst_element_factory_get_type (void)
 {
   static GType elementfactory_type = 0;
 
-  if (!elementfactory_type) {
+  if (G_UNLIKELY (elementfactory_type == 0)) {
     static const GTypeInfo elementfactory_info = {
       sizeof (GstElementFactoryClass),
       NULL,
@@ -83,13 +114,13 @@ gst_element_factory_class_init (GstElementFactoryClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gstpluginfeature_class->unload_thyself =
-      GST_DEBUG_FUNCPTR (gst_element_factory_unload_thyself);
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_element_factory_finalize);
 }
+
 static void
 gst_element_factory_init (GstElementFactory * factory)
 {
-  factory->padtemplates = NULL;
+  factory->staticpadtemplates = NULL;
   factory->numpadtemplates = 0;
 
   factory->uri_type = GST_URI_UNKNOWN;
@@ -98,11 +129,21 @@ gst_element_factory_init (GstElementFactory * factory)
   factory->interfaces = NULL;
 }
 
+static void
+gst_element_factory_finalize (GObject * object)
+{
+  GstElementFactory *factory = GST_ELEMENT_FACTORY (object);
+
+  gst_element_factory_cleanup (factory);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 /**
  * gst_element_factory_find:
  * @name: name of factory to find
  *
- * Search for an element factory of the given name.
+ * Search for an element factory of the given name. Refs the returned
+ * element factory; caller is responsible for unreffing.
  *
  * Returns: #GstElementFactory if found, NULL otherwise
  */
@@ -113,7 +154,8 @@ gst_element_factory_find (const gchar * name)
 
   g_return_val_if_fail (name != NULL, NULL);
 
-  feature = gst_registry_pool_find_feature (name, GST_TYPE_ELEMENT_FACTORY);
+  feature = gst_registry_find_feature (gst_registry_get_default (), name,
+      GST_TYPE_ELEMENT_FACTORY);
   if (feature)
     return GST_ELEMENT_FACTORY (feature);
 
@@ -136,15 +178,15 @@ __gst_element_details_clear (GstElementDetails * dp)
   dp->author = NULL;
 }
 
-#define VALIDATE_SET(__dest, __src, __entry)				\
-G_STMT_START {								\
-  if (g_utf8_validate (__src->__entry, -1, NULL)) {			\
-    __dest->__entry = g_strdup (__src->__entry);			\
-  } else {								\
-    g_warning ("Invalid UTF-8 in " G_STRINGIFY (__entry) ": %s",	\
-        __src->__entry);						\
-    __dest->__entry = g_strdup ("[ERROR: invalid UTF-8]");		\
-  }									\
+#define VALIDATE_SET(__dest, __src, __entry)                            \
+G_STMT_START {                                                          \
+  if (g_utf8_validate (__src->__entry, -1, NULL)) {                     \
+    __dest->__entry = g_strdup (__src->__entry);                        \
+  } else {                                                              \
+    g_warning ("Invalid UTF-8 in " G_STRINGIFY (__entry) ": %s",        \
+        __src->__entry);                                                \
+    __dest->__entry = g_strdup ("[ERROR: invalid UTF-8]");              \
+  }                                                                     \
 } G_STMT_END
 
 void
@@ -168,15 +210,24 @@ __gst_element_details_copy (GstElementDetails * dest,
 static void
 gst_element_factory_cleanup (GstElementFactory * factory)
 {
+  GList *item;
+
   __gst_element_details_clear (&factory->details);
   if (factory->type) {
     g_type_class_unref (g_type_class_peek (factory->type));
     factory->type = 0;
   }
 
-  g_list_foreach (factory->padtemplates, (GFunc) gst_object_unref, NULL);
-  g_list_free (factory->padtemplates);
-  factory->padtemplates = NULL;
+  for (item = factory->staticpadtemplates; item; item = item->next) {
+    GstStaticPadTemplate *templ = item->data;
+
+    g_free (templ->name_template);
+    g_free ((gchar *) templ->static_caps.string);
+    memset (&(templ->static_caps), 0, sizeof (GstStaticCaps));
+    g_free (templ);
+  }
+  g_list_free (factory->staticpadtemplates);
+  factory->staticpadtemplates = NULL;
   factory->numpadtemplates = 0;
   factory->uri_type = GST_URI_UNKNOWN;
   if (factory->uri_protocols) {
@@ -197,7 +248,7 @@ gst_element_factory_cleanup (GstElementFactory * factory)
  * @type: GType of element to register
  *
  * Create a new elementfactory capable of instantiating objects of the
- * given type.
+ * @type and add the factory to @plugin.
  *
  * Returns: TRUE, if the registering succeeded, FALSE on error
  */
@@ -209,31 +260,35 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   GType *interfaces;
   guint n_interfaces, i;
   GstElementClass *klass;
+  GList *item;
 
   g_return_val_if_fail (name != NULL, FALSE);
   g_return_val_if_fail (g_type_is_a (type, GST_TYPE_ELEMENT), FALSE);
 
-  factory = gst_element_factory_find (name);
-
-  if (!factory) {
-    factory =
-        GST_ELEMENT_FACTORY (g_object_new (GST_TYPE_ELEMENT_FACTORY, NULL));
-    gst_plugin_feature_set_name (GST_PLUGIN_FEATURE (factory), name);
-    GST_LOG_OBJECT (factory, "Created new elementfactory for type %s",
-        g_type_name (type));
-    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  } else {
-    g_return_val_if_fail (factory->type == 0, FALSE);
-    gst_element_factory_cleanup (factory);
-    GST_LOG_OBJECT (factory, "Reuse existing elementfactory for type %s",
-        g_type_name (type));
-  }
+  factory = GST_ELEMENT_FACTORY (g_object_new (GST_TYPE_ELEMENT_FACTORY, NULL));
+  gst_plugin_feature_set_name (GST_PLUGIN_FEATURE (factory), name);
+  GST_LOG_OBJECT (factory, "Created new elementfactory for type %s",
+      g_type_name (type));
 
   klass = GST_ELEMENT_CLASS (g_type_class_ref (type));
+  if ((klass->details.longname == NULL) ||
+      (klass->details.klass == NULL) || (klass->details.author == NULL))
+    goto detailserror;
+
   factory->type = type;
   __gst_element_details_copy (&factory->details, &klass->details);
-  factory->padtemplates = g_list_copy (klass->padtemplates);
-  g_list_foreach (factory->padtemplates, (GFunc) gst_object_ref, NULL);
+  for (item = klass->padtemplates; item; item = item->next) {
+    GstPadTemplate *templ = item->data;
+    GstStaticPadTemplate *newt;
+
+    newt = g_new0 (GstStaticPadTemplate, 1);
+    newt->name_template = g_strdup (templ->name_template);
+    newt->direction = templ->direction;
+    newt->presence = templ->presence;
+    newt->static_caps.string = gst_caps_to_string (templ->caps);
+    factory->staticpadtemplates =
+        g_list_append (factory->staticpadtemplates, newt);
+  }
   factory->numpadtemplates = klass->numpadtemplates;
   klass->elementfactory = factory;
 
@@ -243,13 +298,13 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
         g_type_interface_peek (klass, GST_TYPE_URI_HANDLER);
 
     if (!iface || !iface->get_type || !iface->get_protocols)
-      goto error;
+      goto urierror;
     factory->uri_type = iface->get_type ();
     if (!GST_URI_TYPE_IS_VALID (factory->uri_type))
-      goto error;
+      goto urierror;
     factory->uri_protocols = g_strdupv (iface->get_protocols ());
     if (!factory->uri_protocols)
-      goto error;
+      goto urierror;
   }
 
   interfaces = g_type_interfaces (type, &n_interfaces);
@@ -258,13 +313,30 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   }
   g_free (interfaces);
 
+  GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup (plugin->desc.name);
   gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
+  GST_PLUGIN_FEATURE (factory)->loaded = TRUE;
+
+  gst_registry_add_feature (gst_registry_get_default (),
+      GST_PLUGIN_FEATURE (factory));
 
   return TRUE;
 
-error:
-  gst_element_factory_cleanup (factory);
-  return FALSE;
+  /* ERRORS */
+urierror:
+  {
+    GST_WARNING_OBJECT (factory, "error with uri handler!");
+    gst_element_factory_cleanup (factory);
+    return FALSE;
+  }
+
+detailserror:
+  {
+    GST_WARNING_OBJECT (factory,
+        "The GstElementDetails don't seem to have been set properly");
+    gst_element_factory_cleanup (factory);
+    return FALSE;
+  }
 }
 
 /**
@@ -283,50 +355,62 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
 {
   GstElement *element;
   GstElementClass *oclass;
+  GstElementFactory *newfactory;
 
   g_return_val_if_fail (factory != NULL, NULL);
 
-  if (!gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
-    GST_INFO ("could not load element factory for element \"%s\"", name);
-    return NULL;
-  }
+  newfactory =
+      GST_ELEMENT_FACTORY (gst_plugin_feature_load (GST_PLUGIN_FEATURE
+          (factory)));
+
+  if (newfactory == NULL)
+    goto load_failed;
+
+  factory = newfactory;
 
   if (name)
-    GST_INFO ("creating \"%s\" named \"%s\"", GST_PLUGIN_FEATURE_NAME (factory),
-        GST_STR_NULL (name));
+    GST_INFO ("creating element \"%s\" named \"%s\"",
+        GST_PLUGIN_FEATURE_NAME (factory), GST_STR_NULL (name));
   else
-    GST_INFO ("creating \"%s\"", GST_PLUGIN_FEATURE_NAME (factory));
+    GST_INFO ("creating element \"%s\"", GST_PLUGIN_FEATURE_NAME (factory));
 
-  if (factory->type == 0) {
-#ifndef GST_DISABLE_REGISTRY
-    GstPlugin *plugin = GST_PLUGIN_FEATURE (factory)->manager;
+  if (factory->type == 0)
+    goto no_type;
 
-    g_critical
-        ("Factory for `%s' has no type. This probably means the plugin wasn't found because the registry is broken. The plugin GStreamer was looking for is named '%s' and is expected in file '%s'. The registry for this plugin is located at '%s'",
-        GST_PLUGIN_FEATURE_NAME (factory),
-        gst_plugin_get_name (plugin), gst_plugin_get_filename (plugin),
-        GST_IS_XML_REGISTRY (plugin->manager) ? GST_XML_REGISTRY (plugin->
-            manager)->location : "Unknown");
-#else
-    g_critical ("Factory for `%s' has no type",
-        GST_PLUGIN_FEATURE_NAME (factory));
-#endif
-    return NULL;
-  }
+  /* create an instance of the element, cast so we don't assert on NULL */
+  element = GST_ELEMENT_CAST (g_object_new (factory->type, NULL));
+  if (element == NULL)
+    goto no_element;
 
-  oclass = GST_ELEMENT_CLASS (g_type_class_ref (factory->type));
+  /* fill in the pointer to the factory in the element class. The
+   * class will not be unreffed currently. */
+  oclass = GST_ELEMENT_GET_CLASS (element);
   if (oclass->elementfactory == NULL)
     oclass->elementfactory = factory;
 
-  /* create an instance of the element */
-  element = GST_ELEMENT (g_object_new (factory->type, NULL));
-  g_assert (element != NULL);
+  if (name)
+    gst_object_set_name (GST_OBJECT (element), name);
 
-  g_type_class_unref (oclass);
-
-  gst_object_set_name (GST_OBJECT (element), name);
+  GST_DEBUG ("created element \"%s\"", GST_PLUGIN_FEATURE_NAME (factory));
 
   return element;
+
+  /* ERRORS */
+load_failed:
+  {
+    GST_WARNING_OBJECT (factory, "loading plugin returned NULL!");
+    return NULL;
+  }
+no_type:
+  {
+    GST_WARNING_OBJECT (factory, "factory has no type");
+    return NULL;
+  }
+no_element:
+  {
+    GST_WARNING_OBJECT (factory, "could not create element");
+    return NULL;
+  }
 }
 
 /**
@@ -352,42 +436,53 @@ gst_element_factory_make (const gchar * factoryname, const gchar * name)
   GST_LOG ("gstelementfactory: make \"%s\" \"%s\"",
       factoryname, GST_STR_NULL (name));
 
-  /* gst_plugin_load_element_factory (factoryname); */
   factory = gst_element_factory_find (factoryname);
-  if (factory == NULL) {
+  if (factory == NULL)
+    goto no_factory;
+
+  GST_LOG_OBJECT (factory, "found factory %p", factory);
+  element = gst_element_factory_create (factory, name);
+  gst_object_unref (factory);
+  if (element == NULL)
+    goto create_failed;
+
+  return element;
+
+  /* ERRORS */
+no_factory:
+  {
     GST_INFO ("no such element factory \"%s\"!", factoryname);
     return NULL;
   }
-  element = gst_element_factory_create (factory, name);
-  if (element == NULL) {
+create_failed:
+  {
     GST_INFO_OBJECT (factory, "couldn't create instance!");
     return NULL;
   }
-
-  return element;
 }
 
 void
-__gst_element_factory_add_pad_template (GstElementFactory * factory,
-    GstPadTemplate * templ)
+__gst_element_factory_add_static_pad_template (GstElementFactory * factory,
+    GstStaticPadTemplate * templ)
 {
   g_return_if_fail (factory != NULL);
   g_return_if_fail (templ != NULL);
 
-  gst_object_ref (GST_OBJECT (templ));
-  gst_object_sink (GST_OBJECT (templ));
-
-  factory->padtemplates = g_list_append (factory->padtemplates, templ);
+  factory->staticpadtemplates =
+      g_list_append (factory->staticpadtemplates, templ);
   factory->numpadtemplates++;
 }
 
 /**
  * gst_element_factory_get_element_type:
  * @factory: factory to get managed #GType from
- * 
- * Get the #GType for elements managed by this factory
  *
- * Returns: the #GType for elements managed by this factory
+ * Get the #GType for elements managed by this factory. The type can
+ * only be retrieved if the element factory is loaded, which can be
+ * assured with gst_plugin_feature_load().
+ *
+ * Returns: the #GType for elements managed by this factory or 0 if
+ * the factory is not loaded.
  */
 GType
 gst_element_factory_get_element_type (GstElementFactory * factory)
@@ -400,7 +495,7 @@ gst_element_factory_get_element_type (GstElementFactory * factory)
 /**
  * gst_element_factory_get_longname:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the longname for this factory
  *
  * Returns: the longname
@@ -414,9 +509,9 @@ gst_element_factory_get_longname (GstElementFactory * factory)
 }
 
 /**
- * gst_element_factory_get_class:
+ * gst_element_factory_get_klass:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the class for this factory.
  *
  * Returns: the class
@@ -432,7 +527,7 @@ gst_element_factory_get_klass (GstElementFactory * factory)
 /**
  * gst_element_factory_get_description:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the description for this factory.
  *
  * Returns: the description
@@ -448,7 +543,7 @@ gst_element_factory_get_description (GstElementFactory * factory)
 /**
  * gst_element_factory_get_author:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the author for this factory.
  *
  * Returns: the author
@@ -464,7 +559,7 @@ gst_element_factory_get_author (GstElementFactory * factory)
 /**
  * gst_element_factory_get_num_pad_templates:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the number of pad_templates in this factory.
  *
  * Returns: the number of pad_templates
@@ -498,30 +593,30 @@ __gst_element_factory_add_interface (GstElementFactory * elementfactory,
 }
 
 /**
- * gst_element_factory_get_pad_templates:
+ * gst_element_factory_get_static_pad_templates:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the #GList of padtemplates for this factory.
  *
  * Returns: the padtemplates
  */
 G_CONST_RETURN GList *
-gst_element_factory_get_pad_templates (GstElementFactory * factory)
+gst_element_factory_get_static_pad_templates (GstElementFactory * factory)
 {
   g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), NULL);
 
-  return factory->padtemplates;
+  return factory->staticpadtemplates;
 }
 
 /**
  * gst_element_factory_get_uri_type:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets the type of URIs the element supports or GST_URI_UNKNOWN if none.
  *
  * Returns: type of URIs this element supports
  */
-guint
+gint
 gst_element_factory_get_uri_type (GstElementFactory * factory)
 {
   g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), GST_URI_UNKNOWN);
@@ -532,7 +627,7 @@ gst_element_factory_get_uri_type (GstElementFactory * factory)
 /**
  * gst_element_factory_get_uri_protocols:
  * @factory: a #GstElementFactory
- * 
+ *
  * Gets a NULL-terminated array of protocols this element supports or NULL, if
  * no protocols are supported. You may not change the contents of the returned
  * array as it is still ownt by the element factory. Use g_strdupv() if you want to.
@@ -546,83 +641,3 @@ gst_element_factory_get_uri_protocols (GstElementFactory * factory)
 
   return factory->uri_protocols;
 }
-
-/**
- * gst_element_factory_can_src_caps :
- * @factory: factory to query
- * @caps: the caps to check
- *
- * Checks if the factory can source the given capability.
- *
- * Returns: true if it can src the capabilities
- */
-gboolean
-gst_element_factory_can_src_caps (GstElementFactory * factory,
-    const GstCaps * caps)
-{
-  GList *templates;
-
-  g_return_val_if_fail (factory != NULL, FALSE);
-  g_return_val_if_fail (caps != NULL, FALSE);
-
-  templates = factory->padtemplates;
-
-  while (templates) {
-    GstPadTemplate *template = (GstPadTemplate *) templates->data;
-
-    if (template->direction == GST_PAD_SRC) {
-      if (gst_caps_is_always_compatible (GST_PAD_TEMPLATE_CAPS (template),
-              caps))
-        return TRUE;
-    }
-    templates = g_list_next (templates);
-  }
-
-  return FALSE;
-}
-
-/**
- * gst_element_factory_can_sink_caps :
- * @factory: factory to query
- * @caps: the caps to check
- *
- * Checks if the factory can sink the given capability.
- *
- * Returns: true if it can sink the capabilities
- */
-gboolean
-gst_element_factory_can_sink_caps (GstElementFactory * factory,
-    const GstCaps * caps)
-{
-  GList *templates;
-
-  g_return_val_if_fail (factory != NULL, FALSE);
-  g_return_val_if_fail (caps != NULL, FALSE);
-
-  templates = factory->padtemplates;
-
-  while (templates) {
-    GstPadTemplate *template = (GstPadTemplate *) templates->data;
-
-    if (template->direction == GST_PAD_SINK) {
-      if (gst_caps_is_always_compatible (caps,
-              GST_PAD_TEMPLATE_CAPS (template)))
-        return TRUE;
-    }
-    templates = g_list_next (templates);
-  }
-
-  return FALSE;
-}
-static void
-gst_element_factory_unload_thyself (GstPluginFeature * feature)
-{
-  GstElementFactory *factory;
-
-  factory = GST_ELEMENT_FACTORY (feature);
-
-  if (factory->type) {
-    g_type_class_unref (g_type_class_peek (factory->type));
-    factory->type = 0;
-  }
-}
