@@ -21,32 +21,101 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstinfo
+ * @short_description: Debugging and logging facillities
+ * @see_also: #GstConfig, #Gst for command line parameters
+ * and environment variables that affect the debugging output.
+ *
+ * GStreamer's debugging subsystem is an easy way to get information about what
+ * the application is doing.  It is not meant for programming errors. Use GLib
+ * methods (g_warning and friends) for that.
+ *
+ * The debugging subsystem works only after GStreamer has been initialized
+ * - for example by calling gst_init().
+ *
+ * The debugging subsystem is used to log informational messages while the
+ * application runs.  Each messages has some properties attached to it. Among
+ * these properties are the debugging category, the severity (called "level"
+ * here) and an optional #GObject it belongs to. Each of these messages is sent
+ * to all registered debugging handlers, which then handle the messages.
+ * GStreamer attaches a default handler on startup, which outputs requested
+ * messages to stderr.
+ *
+ * Messages are output by using shortcut macros like #GST_DEBUG,
+ * #GST_CAT_ERROR_OBJECT or similar. These all expand to calling gst_debug_log()
+ * with the right parameters.
+ * The only thing a developer will probably want to do is define his own
+ * categories. This is easily done with 3 lines. At the top of your code,
+ * declare
+ * the variables and set the default category.
+ * <informalexample>
+ * <programlisting>
+ * GST_DEBUG_CATEGORY_STATIC (my_category);     // define category (statically)
+ * &hash;define GST_CAT_DEFAULT my_category     // set as default
+ * </programlisting>
+ * </informalexample>
+ * After that you only need to initialize the category.
+ * <informalexample>
+ * <programlisting>
+ * GST_DEBUG_CATEGORY_INIT (my_category, "my category",
+ *                          0, "This is my very own");
+ * </programlisting>
+ * </informalexample>
+ * Initialization must be done before the category is used first.
+ * Plugins do this
+ * in their plugin_init function, libraries and applications should do that
+ * during their initialization.
+ *
+ * The whole debugging subsystem can be disabled at build time with passing the
+ * --disable-gst-debug switch to configure. If this is done, every function,
+ * macro and even structs described in this file evaluate to default values or
+ * nothing at all.
+ * So don't take addresses of these functions or use other tricks.
+ * If you must do that for some reason, there is still an option.
+ * If the debugging
+ * subsystem was compiled out, #GST_DISABLE_GST_DEBUG is defined in
+ * &lt;gst/gst.h&gt;,
+ * so you can check that before doing your trick.
+ * Disabling the debugging subsystem will give you a slight (read: unnoticable)
+ * speed increase and will reduce the size of your compiled code. The GStreamer
+ * library itself becomes around 10% smaller.
+ *
+ * Please note that there are naming conventions for the names of debugging
+ * categories. These are explained at GST_DEBUG_CATEGORY_INIT().
+ */
+
 #include "gst_private.h"
 #include "gstinfo.h"
 
 #ifndef GST_DISABLE_GST_DEBUG
 
 #ifdef HAVE_DLFCN_H
-#include <dlfcn.h>
+#  include <dlfcn.h>
 #endif
 #ifdef HAVE_PRINTF_EXTENSION
-#include <printf.h>
+#  include <printf.h>
 #endif
 #include <stdio.h>              /* fprintf */
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#  include <unistd.h>           /* getpid on UNIX */
+#endif
+#ifdef HAVE_PROCESS_H
+#  include <process.h>          /* getpid on win32 */
 #endif
 #include <string.h>             /* G_VA_COPY */
-#include "gstelement.h"
-#include "gstpad.h"
-#include "gstscheduler.h"
 #include "gst_private.h"
+#include "gstutils.h"
+#include "gstsegment.h"
 #ifdef HAVE_VALGRIND
-#include <valgrind/valgrind.h>
+#  include <valgrind/valgrind.h>
 #endif
+#include <glib/gprintf.h>       /* g_sprintf */
 
 /* underscore is to prevent conflict with GST_CAT_DEBUG define */
 GST_DEBUG_CATEGORY_STATIC (_GST_CAT_DEBUG);
+/* time of initialization, so we get useful debugging output times */
+static GstClockTime start_time;
 
 #if 0
 #if defined __sgi__
@@ -81,7 +150,9 @@ static void gst_debug_reset_threshold (gpointer category, gpointer unused);
 static void gst_debug_reset_all_thresholds (void);
 
 #ifdef HAVE_PRINTF_EXTENSION
-static int _gst_info_printf_extension (FILE * stream,
+static int _gst_info_printf_extension_ptr (FILE * stream,
+    const struct printf_info *info, const void *const *args);
+static int _gst_info_printf_extension_segment (FILE * stream,
     const struct printf_info *info, const void *const *args);
 static int _gst_info_printf_extension_arginfo (const struct printf_info *info,
     size_t n, int *argtypes);
@@ -118,26 +189,25 @@ LogFuncEntry;
 static GStaticMutex __log_func_mutex = G_STATIC_MUTEX_INIT;
 static GSList *__log_functions = NULL;
 
-static GstAtomicInt __default_level;
-static GstAtomicInt __use_color;
-gboolean __gst_debug_enabled = TRUE;
+static gint __default_level;
+static gint __use_color;
 
+/* disabled by default, as soon as some threshold is set > NONE,
+ * it becomes enabled. */
+gboolean __gst_debug_enabled = FALSE;
+GstDebugLevel __gst_debug_min = GST_LEVEL_NONE;
 
 GstDebugCategory *GST_CAT_DEFAULT = NULL;
 
 GstDebugCategory *GST_CAT_GST_INIT = NULL;
-GstDebugCategory *GST_CAT_COTHREADS = NULL;
-GstDebugCategory *GST_CAT_COTHREAD_SWITCH = NULL;
 GstDebugCategory *GST_CAT_AUTOPLUG = NULL;
 GstDebugCategory *GST_CAT_AUTOPLUG_ATTEMPT = NULL;
 GstDebugCategory *GST_CAT_PARENTAGE = NULL;
 GstDebugCategory *GST_CAT_STATES = NULL;
-GstDebugCategory *GST_CAT_PLANNING = NULL;
 GstDebugCategory *GST_CAT_SCHEDULING = NULL;
 
-/* FIXME: remove GST_CAT_DATAFLOW in 0.9 */
-GstDebugCategory *GST_CAT_DATAFLOW = NULL;
 GstDebugCategory *GST_CAT_BUFFER = NULL;
+GstDebugCategory *GST_CAT_BUS = NULL;
 GstDebugCategory *GST_CAT_CAPS = NULL;
 GstDebugCategory *GST_CAT_CLOCK = NULL;
 GstDebugCategory *GST_CAT_ELEMENT_PADS = NULL;
@@ -146,18 +216,19 @@ GstDebugCategory *GST_CAT_PIPELINE = NULL;
 GstDebugCategory *GST_CAT_PLUGIN_LOADING = NULL;
 GstDebugCategory *GST_CAT_PLUGIN_INFO = NULL;
 GstDebugCategory *GST_CAT_PROPERTIES = NULL;
-GstDebugCategory *GST_CAT_THREAD = NULL;
 GstDebugCategory *GST_CAT_TYPES = NULL;
 GstDebugCategory *GST_CAT_XML = NULL;
 GstDebugCategory *GST_CAT_NEGOTIATION = NULL;
 GstDebugCategory *GST_CAT_REFCOUNTING = NULL;
 GstDebugCategory *GST_CAT_ERROR_SYSTEM = NULL;
 GstDebugCategory *GST_CAT_EVENT = NULL;
+GstDebugCategory *GST_CAT_MESSAGE = NULL;
 GstDebugCategory *GST_CAT_PARAMS = NULL;
 GstDebugCategory *GST_CAT_CALL_TRACE = NULL;
-GstDebugCategory *GST_CAT_SEEK = NULL;
 GstDebugCategory *GST_CAT_SIGNAL = NULL;
 GstDebugCategory *GST_CAT_PROBE = NULL;
+GstDebugCategory *GST_CAT_REGISTRY = NULL;
+GstDebugCategory *GST_CAT_QOS = NULL;
 
 /* FIXME: export this? */
 gboolean
@@ -175,11 +246,9 @@ __gst_in_valgrind (void)
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
       GST_CAT_INFO (GST_CAT_GST_INIT, "we're running inside valgrind");
-      VALGRIND_PRINTF
-          ("GStreamer has detected that it is running inside valgrind.");
-      VALGRIND_PRINTF
-          ("It might now take different code paths to ease debugging.");
-      VALGRIND_PRINTF ("Of course, this may also lead to different bugs.");
+      printf ("GStreamer has detected that it is running inside valgrind.\n");
+      printf ("It might now take different code paths to ease debugging.\n");
+      printf ("Of course, this may also lead to different bugs.\n");
       in_valgrind = GST_VG_INSIDE;
     } else {
       GST_CAT_LOG (GST_CAT_GST_INIT, "not doing extra valgrind stuff");
@@ -196,19 +265,27 @@ __gst_in_valgrind (void)
 
 /**
  * _gst_debug_init:
- * 
+ *
  * Initializes the debugging system.
- * Normally you don't want to call this, because gst_init does it for you.
+ * Normally you don't want to call this, because gst_init() does it for you.
  */
 void
 _gst_debug_init (void)
 {
-  gst_atomic_int_init (&__default_level, GST_LEVEL_DEFAULT);
-  gst_atomic_int_init (&__use_color, 1);
+  GTimeVal current;
+
+  gst_atomic_int_set (&__default_level, GST_LEVEL_DEFAULT);
+  gst_atomic_int_set (&__use_color, 1);
+
+  /* get time we started for debugging messages */
+  g_get_current_time (&current);
+  start_time = GST_TIMEVAL_TO_TIME (current);
 
 #ifdef HAVE_PRINTF_EXTENSION
-  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension,
+  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension_ptr,
       _gst_info_printf_extension_arginfo);
+  register_printf_function (GST_SEGMENT_FORMAT[0],
+      _gst_info_printf_extension_segment, _gst_info_printf_extension_arginfo);
 #endif
 
   /* do NOT use a single debug function before this line has been run */
@@ -222,10 +299,6 @@ _gst_debug_init (void)
   /* FIXME: add descriptions here */
   GST_CAT_GST_INIT = _gst_debug_category_new ("GST_INIT",
       GST_DEBUG_BOLD | GST_DEBUG_FG_RED, NULL);
-  GST_CAT_COTHREADS = _gst_debug_category_new ("GST_COTHREADS",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, NULL);
-  GST_CAT_COTHREAD_SWITCH = _gst_debug_category_new ("GST_COTHREAD_SWITCH",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_GREEN, NULL);
   GST_CAT_AUTOPLUG = _gst_debug_category_new ("GST_AUTOPLUG",
       GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE, NULL);
   GST_CAT_AUTOPLUG_ATTEMPT = _gst_debug_category_new ("GST_AUTOPLUG_ATTEMPT",
@@ -234,15 +307,11 @@ _gst_debug_init (void)
       GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED, NULL);
   GST_CAT_STATES = _gst_debug_category_new ("GST_STATES",
       GST_DEBUG_BOLD | GST_DEBUG_FG_RED, NULL);
-  GST_CAT_PLANNING = _gst_debug_category_new ("GST_PLANNING",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_MAGENTA, NULL);
   GST_CAT_SCHEDULING = _gst_debug_category_new ("GST_SCHEDULING",
       GST_DEBUG_BOLD | GST_DEBUG_FG_MAGENTA, NULL);
-/* FIXME: remove GST_CAT_DATAFLOW in 0.9 */
-  GST_CAT_DATAFLOW = _gst_debug_category_new ("GST_DATAFLOW",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, "dataflow inside pads");
   GST_CAT_BUFFER = _gst_debug_category_new ("GST_BUFFER",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, NULL);
+      GST_DEBUG_BOLD | GST_DEBUG_BG_GREEN, NULL);
+  GST_CAT_BUS = _gst_debug_category_new ("GST_BUS", GST_DEBUG_BG_YELLOW, NULL);
   GST_CAT_CAPS = _gst_debug_category_new ("GST_CAPS",
       GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE, NULL);
   GST_CAT_CLOCK = _gst_debug_category_new ("GST_CLOCK",
@@ -259,8 +328,6 @@ _gst_debug_init (void)
       GST_DEBUG_BOLD | GST_DEBUG_FG_CYAN, NULL);
   GST_CAT_PROPERTIES = _gst_debug_category_new ("GST_PROPERTIES",
       GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_BLUE, NULL);
-  GST_CAT_THREAD = _gst_debug_category_new ("GST_THREAD",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_RED, NULL);
   GST_CAT_TYPES = _gst_debug_category_new ("GST_TYPES",
       GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED, NULL);
   GST_CAT_XML = _gst_debug_category_new ("GST_XML",
@@ -268,23 +335,24 @@ _gst_debug_init (void)
   GST_CAT_NEGOTIATION = _gst_debug_category_new ("GST_NEGOTIATION",
       GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE, NULL);
   GST_CAT_REFCOUNTING = _gst_debug_category_new ("GST_REFCOUNTING",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE | GST_DEBUG_BG_GREEN, NULL);
+      GST_DEBUG_BOLD | GST_DEBUG_FG_RED | GST_DEBUG_BG_BLUE, NULL);
   GST_CAT_ERROR_SYSTEM = _gst_debug_category_new ("GST_ERROR_SYSTEM",
       GST_DEBUG_BOLD | GST_DEBUG_FG_RED | GST_DEBUG_BG_WHITE, NULL);
 
   GST_CAT_EVENT = _gst_debug_category_new ("GST_EVENT",
       GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE, NULL);
+  GST_CAT_MESSAGE = _gst_debug_category_new ("GST_MESSAGE",
+      GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED, NULL);
   GST_CAT_PARAMS = _gst_debug_category_new ("GST_PARAMS",
       GST_DEBUG_BOLD | GST_DEBUG_FG_BLACK | GST_DEBUG_BG_YELLOW, NULL);
   GST_CAT_CALL_TRACE = _gst_debug_category_new ("GST_CALL_TRACE",
       GST_DEBUG_BOLD, NULL);
-  /* FIXME: fold back to GST_CAT_EVENT in 0.9 */
-  GST_CAT_SEEK = _gst_debug_category_new ("GST_SEEK",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE, "plugins reacting to seek events");
   GST_CAT_SIGNAL = _gst_debug_category_new ("GST_SIGNAL",
       GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED, NULL);
   GST_CAT_PROBE = _gst_debug_category_new ("GST_PROBE",
       GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, "pad probes");
+  GST_CAT_REGISTRY = _gst_debug_category_new ("GST_REGISTRY", 0, "registry");
+  GST_CAT_QOS = _gst_debug_category_new ("GST_QOS", 0, "QoS");
 
 
   /* print out the valgrind message if we're in valgrind */
@@ -304,7 +372,7 @@ _gst_debug_init (void)
  * @object: the object this message relates to or NULL if none
  * @format: a printf style format string
  * @...: optional arguments for the format
- * 
+ *
  * Logs the given message using the currently registered debugging handlers.
  */
 void
@@ -330,7 +398,7 @@ gst_debug_log (GstDebugCategory * category, GstDebugLevel level,
  * @object: the object this message relates to or NULL if none
  * @format: a printf style format string
  * @args: optional arguments for the format
- * 
+ *
  * Logs the given message using the currently registered debugging handlers.
  */
 void
@@ -423,18 +491,76 @@ gst_debug_print_object (gpointer ptr)
   if (G_IS_OBJECT (object)) {
     return g_strdup_printf ("<%s@%p>", G_OBJECT_TYPE_NAME (object), object);
   }
+  if (GST_IS_MESSAGE (object)) {
+    GstMessage *msg = GST_MESSAGE_CAST (object);
+    gchar *s, *ret;
+
+    if (msg->structure) {
+      s = gst_structure_to_string (msg->structure);
+    } else {
+      s = g_strdup ("(NULL)");
+    }
+
+    ret = g_strdup_printf ("%s message from element '%s': %s",
+        GST_MESSAGE_TYPE_NAME (msg), (msg->src != NULL) ?
+        GST_ELEMENT_NAME (msg->src) : "(NULL)", s);
+    g_free (s);
+    return ret;
+  }
 
   return g_strdup_printf ("%p", ptr);
 }
 
+static gchar *
+gst_debug_print_segment (gpointer ptr)
+{
+  GstSegment *segment = (GstSegment *) ptr;
+
+  /* nicely printed segment */
+  if (segment == NULL) {
+    return g_strdup ("(NULL)");
+  }
+
+  switch (segment->format) {
+    case GST_FORMAT_UNDEFINED:{
+      return g_strdup_printf ("UNDEFINED segment");
+    }
+    case GST_FORMAT_TIME:{
+      return g_strdup_printf ("time segment start=%" GST_TIME_FORMAT
+          ", stop=%" GST_TIME_FORMAT ", last_stop=%" GST_TIME_FORMAT
+          ", duration=%" GST_TIME_FORMAT ", rate=%f, applied_rate=%f"
+          ", flags=0x%02x, time=%" GST_TIME_FORMAT ", accum=%" GST_TIME_FORMAT,
+          GST_TIME_ARGS (segment->start), GST_TIME_ARGS (segment->stop),
+          GST_TIME_ARGS (segment->last_stop), GST_TIME_ARGS (segment->duration),
+          segment->rate, segment->applied_rate, (guint) segment->flags,
+          GST_TIME_ARGS (segment->time), GST_TIME_ARGS (segment->accum));
+    }
+    default:{
+      const gchar *format_name;
+
+      format_name = gst_format_get_name (segment->format);
+      if (G_UNLIKELY (format_name == NULL))
+        format_name = "(UNKNOWN FORMAT)";
+      return g_strdup_printf ("%s segment start=%" G_GINT64_FORMAT
+          ", stop=%" G_GINT64_FORMAT ", last_stop=%" G_GINT64_FORMAT
+          ", duration=%" G_GINT64_FORMAT ", rate=%f, applied_rate=%f"
+          ", flags=0x%02x, time=%" GST_TIME_FORMAT ", accum=%" GST_TIME_FORMAT,
+          format_name, segment->start, segment->stop, segment->last_stop,
+          segment->duration, segment->rate, segment->applied_rate,
+          (guint) segment->flags, GST_TIME_ARGS (segment->time),
+          GST_TIME_ARGS (segment->accum));
+    }
+  }
+}
+
 /**
  * gst_debug_construct_term_color:
  * @colorinfo: the color info
- * 
+ *
  * Constructs a string that can be used for getting the desired color in color
  * terminals.
  * You need to free the string after use.
- * 
+ *
  * Returns: a string containing the color definition
  */
 gchar *
@@ -475,25 +601,37 @@ gst_debug_construct_term_color (guint colorinfo)
  * @message: the actual message
  * @object: the object this message relates to or NULL if none
  * @unused: an unused variable, reserved for some user_data.
- * 
+ *
  * The default logging handler used by GStreamer. Logging functions get called
  * whenever a macro like GST_DEBUG or similar is used. This function outputs the
  * message and additional info using the glib error handler.
- * You can add other handlers by using #gst_debug_add_log_function. 
+ * You can add other handlers by using gst_debug_add_log_function().
  * And you can remove this handler by calling
- * gst_debug_remove_log_function (gst_debug_log_default);
+ * gst_debug_remove_log_function(gst_debug_log_default);
  */
 void
 gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
     const gchar * file, const gchar * function, gint line,
     GObject * object, GstDebugMessage * message, gpointer unused)
 {
-  gchar *color;
+  gchar *color = NULL;
   gchar *clear;
-  gchar *obj;
-  gchar *pidcolor;
+  gchar *obj = NULL;
+  gchar pidcolor[10];
+  const gchar *levelcolor;
   gint pid;
   GTimeVal now;
+  GstClockTime elapsed;
+  gboolean free_color = TRUE;
+  gboolean free_obj = TRUE;
+  static const gchar *levelcolormap[] = {
+    "\033[37m",                 /* GST_LEVEL_NONE */
+    "\033[31;01m",              /* GST_LEVEL_ERROR */
+    "\033[33;01m",              /* GST_LEVEL_WARNING */
+    "\033[32;01m",              /* GST_LEVEL_INFO */
+    "\033[36m",                 /* GST_LEVEL_DEBUG */
+    "\033[37m"                  /* GST_LEVEL_LOG */
+  };
 
   if (level > gst_debug_category_get_threshold (category))
     return;
@@ -502,38 +640,56 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
 
   /* color info */
   if (gst_debug_is_colored ()) {
-    color =
-        gst_debug_construct_term_color (gst_debug_category_get_color
+    color = gst_debug_construct_term_color (gst_debug_category_get_color
         (category));
     clear = "\033[00m";
-    pidcolor = g_strdup_printf ("\033[3%1dm", pid % 6 + 31);
+    g_sprintf (pidcolor, "\033[3%1dm", pid % 6 + 31);
+    levelcolor = levelcolormap[level];
   } else {
-    color = g_strdup ("");
+    color = "\0";
+    free_color = FALSE;
     clear = "";
-    pidcolor = g_strdup ("");
+    pidcolor[0] = '\0';
+    levelcolor = "\0";
   }
 
-  obj = object ? gst_debug_print_object (object) : g_strdup ("");
+  if (object) {
+    obj = gst_debug_print_object (object);
+  } else {
+    obj = "\0";
+    free_obj = FALSE;
+  }
 
   g_get_current_time (&now);
-  g_printerr ("%s (%p - %" GST_TIME_FORMAT
-      ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
-      gst_debug_level_get_name (level), g_thread_self (),
-      GST_TIME_ARGS (GST_TIMEVAL_TO_TIME (now)), color,
-      gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
-      color, file, line, function, obj, clear, gst_debug_message_get (message));
+  elapsed = GST_TIMEVAL_TO_TIME (now) - start_time;
 
-  g_free (color);
-  g_free (pidcolor);
-  g_free (obj);
+  /*
+     g_printerr ("%s (%p - %" GST_TIME_FORMAT ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
+     gst_debug_level_get_name (level), g_thread_self (),
+     GST_TIME_ARGS (elapsed), color,
+     gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
+     color, file, line, function, obj, clear, gst_debug_message_get (message));
+   */
+
+  g_printerr ("%" GST_TIME_FORMAT
+      " %s%5d%s %p %s%s%s %s%20s %s:%d:%s:%s%s %s\n", GST_TIME_ARGS (elapsed),
+      pidcolor, pid, clear, g_thread_self (), levelcolor,
+      gst_debug_level_get_name (level), clear, color,
+      gst_debug_category_get_name (category), file, line, function, obj, clear,
+      gst_debug_message_get (message));
+
+  if (free_color)
+    g_free (color);
+  if (free_obj)
+    g_free (obj);
 }
 
 /**
  * gst_debug_level_get_name:
  * @level: the level to get the name for
- * 
+ *
  * Get the string trepresentation of a debugging level
- * 
+ *
  * Returns: the name
  */
 const gchar *
@@ -562,7 +718,7 @@ gst_debug_level_get_name (GstDebugLevel level)
  * gst_debug_add_log_function:
  * @func: the function to use
  * @data: user data
- * 
+ *
  * Adds the logging function to the list of logging functions.
  * Be sure to use G_GNUC_NO_INSTRUMENT on that function, it is needed.
  */
@@ -580,7 +736,7 @@ gst_debug_add_log_function (GstLogFunction func, gpointer data)
   /* FIXME: we leak the old list here - other threads might access it right now
    * in gst_debug_logv. Another solution is to lock the mutex in gst_debug_logv,
    * but that is waaay costly.
-   * It'd probably be clever to use some kind of RCU here, but I don't know 
+   * It'd probably be clever to use some kind of RCU here, but I don't know
    * anything about that.
    */
   g_static_mutex_lock (&__log_func_mutex);
@@ -636,9 +792,9 @@ gst_debug_remove_with_compare_func (GCompareFunc func, gpointer data)
 /**
  * gst_debug_remove_log_function:
  * @func: the log function to remove
- * 
+ *
  * Removes all registrered instances of the given logging functions.
- * 
+ *
  * Returns: How many instances of the function were removed
  */
 guint
@@ -660,9 +816,9 @@ gst_debug_remove_log_function (GstLogFunction func)
 /**
  * gst_debug_remove_log_function_by_data:
  * @data: user data of the log function to remove
- * 
+ *
  * Removes all registrered instances of log functions with the given user data.
- * 
+ *
  * Returns: How many instances of the function were removed
  */
 guint
@@ -683,7 +839,7 @@ gst_debug_remove_log_function_by_data (gpointer data)
 /**
  * gst_debug_set_colored:
  * @colored: Whether to use colored output or not
- * 
+ *
  * Sets or unsets the use of coloured debugging output.
  */
 void
@@ -694,21 +850,21 @@ gst_debug_set_colored (gboolean colored)
 
 /**
  * gst_debug_is_colored:
- * 
+ *
  * Checks if the debugging output should be colored.
- * 
+ *
  * Returns: TRUE, if the debug output should be colored.
  */
 gboolean
 gst_debug_is_colored (void)
 {
-  return gst_atomic_int_read (&__use_color) == 0 ? FALSE : TRUE;
+  return g_atomic_int_get (&__use_color) == 0 ? FALSE : TRUE;
 }
 
 /**
  * gst_debug_set_active:
  * @active: Whether to use debugging output or not
- * 
+ *
  * If activated, debugging messages are sent to the debugging
  * handlers.
  * It makes sense to deactivate it for speed issues.
@@ -719,13 +875,17 @@ void
 gst_debug_set_active (gboolean active)
 {
   __gst_debug_enabled = active;
+  if (active)
+    __gst_debug_min = GST_LEVEL_COUNT;
+  else
+    __gst_debug_min = GST_LEVEL_NONE;
 }
 
 /**
  * gst_debug_is_active:
- * 
+ *
  * Checks if debugging output is activated.
- * 
+ *
  * Returns: TRUE, if debugging is activated
  */
 gboolean
@@ -737,7 +897,7 @@ gst_debug_is_active (void)
 /**
  * gst_debug_set_default_threshold:
  * @level: level to set
- * 
+ *
  * Sets the default threshold to the given level and updates all categories to
  * use this threshold.
  */
@@ -750,15 +910,15 @@ gst_debug_set_default_threshold (GstDebugLevel level)
 
 /**
  * gst_debug_get_default_threshold:
- * 
+ *
  * Returns the default threshold that is used for new categories.
- * 
+ *
  * Returns: the default threshold level
  */
 GstDebugLevel
 gst_debug_get_default_threshold (void)
 {
-  return (GstDebugLevel) gst_atomic_int_read (&__default_level);
+  return (GstDebugLevel) g_atomic_int_get (&__default_level);
 }
 static void
 gst_debug_reset_threshold (gpointer category, gpointer unused)
@@ -881,8 +1041,7 @@ _gst_debug_category_new (gchar * name, guint color, gchar * description)
   } else {
     cat->description = g_strdup ("no description");
   }
-  cat->threshold = g_new (GstAtomicInt, 1);
-  gst_atomic_int_init (cat->threshold, 0);
+  gst_atomic_int_set (&cat->threshold, 0);
   gst_debug_reset_threshold (cat, NULL);
 
   /* add to category list */
@@ -912,8 +1071,6 @@ gst_debug_category_free (GstDebugCategory * category)
 
   g_free ((gpointer) category->name);
   g_free ((gpointer) category->description);
-  gst_atomic_int_destroy (category->threshold);
-  g_free (category->threshold);
   g_free (category);
 }
 
@@ -937,7 +1094,12 @@ gst_debug_category_set_threshold (GstDebugCategory * category,
 {
   g_return_if_fail (category != NULL);
 
-  gst_atomic_int_set (category->threshold, level);
+  if (level > __gst_debug_min) {
+    __gst_debug_enabled = TRUE;
+    __gst_debug_min = level;
+  }
+
+  gst_atomic_int_set (&category->threshold, level);
 }
 
 /**
@@ -967,7 +1129,7 @@ gst_debug_category_reset_threshold (GstDebugCategory * category)
 GstDebugLevel
 gst_debug_category_get_threshold (GstDebugCategory * category)
 {
-  return gst_atomic_int_read (category->threshold);
+  return g_atomic_int_get (&category->threshold);
 }
 
 /**
@@ -1019,8 +1181,6 @@ gst_debug_category_get_description (GstDebugCategory * category)
  * Returns a snapshot of a all categories that are currently in use . This list
  * may change anytime.
  * The caller has to free the list after use.
- * <emphasis>This function is not threadsafe, so only use it while only the
- * main thread is running.</emphasis>
  *
  * Returns: the list of categories
  */
@@ -1042,24 +1202,30 @@ static GHashTable *__gst_function_pointers;     /* NULL */
 static GStaticMutex __dbg_functions_mutex = G_STATIC_MUTEX_INIT;
 
 const gchar *
-_gst_debug_nameof_funcptr (void *ptr)
+_gst_debug_nameof_funcptr (GstDebugFuncPtr ptr)
     G_GNUC_NO_INSTRUMENT;
 
 /* This function MUST NOT return NULL */
-     const gchar *_gst_debug_nameof_funcptr (void *ptr)
+     const gchar *_gst_debug_nameof_funcptr (GstDebugFuncPtr func)
 {
+  gpointer ptr = (gpointer) func;
   gchar *ptrname;
 
 #ifdef HAVE_DLADDR
   Dl_info dlinfo;
 #endif
 
-  if (__gst_function_pointers) {
-    g_static_mutex_lock (&__dbg_functions_mutex);
+  if (G_UNLIKELY (func == NULL))
+    return "(NULL)";
+
+  g_static_mutex_lock (&__dbg_functions_mutex);
+  if (G_LIKELY (__gst_function_pointers)) {
     ptrname = g_hash_table_lookup (__gst_function_pointers, ptr);
     g_static_mutex_unlock (&__dbg_functions_mutex);
-    if (ptrname)
+    if (G_LIKELY (ptrname))
       return ptrname;
+  } else {
+    g_static_mutex_unlock (&__dbg_functions_mutex);
   }
   /* we need to create an entry in the hash table for this one so we don't leak
    * the name */
@@ -1079,9 +1245,11 @@ _gst_debug_nameof_funcptr (void *ptr)
   }
 }
 
-void *
-_gst_debug_register_funcptr (void *ptr, gchar * ptrname)
+void
+_gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
 {
+  gpointer ptr = (gpointer) func;
+
   g_static_mutex_lock (&__dbg_functions_mutex);
 
   if (!__gst_function_pointers)
@@ -1090,13 +1258,11 @@ _gst_debug_register_funcptr (void *ptr, gchar * ptrname)
     g_hash_table_insert (__gst_function_pointers, ptr, ptrname);
 
   g_static_mutex_unlock (&__dbg_functions_mutex);
-
-  return ptr;
 }
 
 #ifdef HAVE_PRINTF_EXTENSION
 static int
-_gst_info_printf_extension (FILE * stream, const struct printf_info *info,
+_gst_info_printf_extension_ptr (FILE * stream, const struct printf_info *info,
     const void *const *args)
 {
   char *buffer;
@@ -1110,7 +1276,26 @@ _gst_info_printf_extension (FILE * stream, const struct printf_info *info,
   len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
       buffer);
 
-  free (buffer);
+  g_free (buffer);
+  return len;
+}
+
+static int
+_gst_info_printf_extension_segment (FILE * stream,
+    const struct printf_info *info, const void *const *args)
+{
+  char *buffer;
+  int len;
+  void *ptr;
+
+  buffer = NULL;
+  ptr = *(void **) args[0];
+
+  buffer = gst_debug_print_segment (ptr);
+  len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
+      buffer);
+
+  g_free (buffer);
   return len;
 }
 
@@ -1182,6 +1367,12 @@ __cyg_profile_func_exit (void *this_fn, void *call_site)
   g_free (name);
 }
 
+/**
+ * gst_debug_print_stack_trace:
+ *
+ * If GST_ENABLE_FUNC_INSTRUMENTATION is defined a stacktracke is available for
+ * gstreamer code, which can be printed with this function.
+ */
 void
 gst_debug_print_stack_trace (void)
 {
