@@ -28,6 +28,63 @@
  *
  * One registry holds the metadata of a set of plugins.
  * All registries build the #GstRegistryPool.
+ * 
+ * <emphasis role="bold">Design:</emphasis>
+ *
+ * The #GstRegistry object is a list of plugins and some functions for dealing
+ * with them. #GstPlugins are matched 1-1 with a file on disk, and may or may
+ * not be loaded at a given time. There may be multiple #GstRegistry objects,
+ * but the "default registry" is the only object that has any meaning to the
+ * core.
+ *
+ * The registry.xml file is actually a cache of plugin information. This is
+ * unlike versions prior to 0.10, where the registry file was the primary source
+ * of plugin information, and was created by the gst-register command.
+ *
+ * The primary source, at all times, of plugin information is each plugin file
+ * itself. Thus, if an application wants information about a particular plugin,
+ * or wants to search for a feature that satisfies given criteria, the primary
+ * means of doing so is to load every plugin and look at the resulting
+ * information that is gathered in the default registry. Clearly, this is a time
+ * consuming process, so we cache information in the registry.xml file.
+ *
+ * On startup, plugins are searched for in the plugin search path. This path can
+ * be set directly using the %GST_PLUGIN_PATH environment variable. The registry
+ * file is loaded from ~/.gstreamer-$GST_MAJORMINOR/registry-$ARCH.xml or the
+ * file listed in the %GST_REGISTRY env var. The only reason to change the
+ * registry location is for testing.
+ *
+ * For each plugin that is found in the plugin search path, there could be 3
+ * possibilities for cached information:
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>the cache may not contain information about a given file.</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>the cache may have stale information.</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>the cache may have current information.</para>
+ *   </listitem>
+ * </itemizedlist>
+ *
+ * In the first two cases, the plugin is loaded and the cache updated. In
+ * addition to these cases, the cache may have entries for plugins that are not
+ * relevant to the current process. These are marked as not available to the
+ * current process. If the cache is updated for whatever reason, it is marked
+ * dirty.
+ *
+ * A dirty cache is written out at the end of initialization. Each entry is
+ * checked to make sure the information is minimally valid. If not, the entry is
+ * simply dropped.
+ *
+ * <emphasis role="bold">Implementation notes:</emphasis>
+ *
+ * The "cache" and "default registry" are different concepts and can represent
+ * different sets of plugins. For various reasons, at init time, the cache is
+ * stored in the default registry, and plugins not relevant to the current
+ * process are marked with the %GST_PLUGIN_FLAG_CACHED bit. These plugins are
+ * removed at the end of intitialization.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,58 +114,6 @@
 static GStaticMutex _gst_registry_mutex = G_STATIC_MUTEX_INIT;
 static GstRegistry *_gst_registry_default = NULL;
 
-/*
- * Design:
- *
- * The GstRegistry object is a list of plugins and some functions for dealing
- * with them. Plugins are matched 1-1 with a file on disk, and may or may not be
- * loaded at a given time. There may be multiple GstRegistry objects, but the
- * "default registry" is the only object that has any meaning to the core.
- *
- * The registry.xml file is actually a cache of plugin information. This is
- * unlike versions prior to 0.10, where the registry file was the primary source
- * of plugin information, and was created by the gst-register command.
- *
- * The primary source, at all times, of plugin information is each plugin file
- * itself. Thus, if an application wants information about a particular plugin,
- * or wants to search for a feature that satisfies given criteria, the primary
- * means of doing so is to load every plugin and look at the resulting
- * information that is gathered in the default registry. Clearly, this is a time
- * consuming process, so we cache information in the registry.xml file.
- *
- * On startup, plugins are searched for in the plugin search path. This path can
- * be set directly using the GST_PLUGIN_PATH environment variable. The registry
- * file is loaded from ~/.gstreamer-$GST_MAJORMINOR/registry-$ARCH.xml or the
- * file listed in the GST_REGISTRY env var. The only reason to change the
- * registry location is for testing.
- *
- * For each plugin that is found in the plugin search path, there could be 3
- * possibilities for cached information:
- *
- *  - the cache may not contain information about a given file.
- *  - the cache may have stale information.
- *  - the cache may have current information.
- *
- * In the first two cases, the plugin is loaded and the cache updated. In
- * addition to these cases, the cache may have entries for plugins that are not
- * relevant to the current process. These are marked as not available to the
- * current process. If the cache is updated for whatever reason, it is marked
- * dirty.
- *
- * A dirty cache is written out at the end of initialization. Each entry is
- * checked to make sure the information is minimally valid. If not, the entry is
- * simply dropped.
- *
- * Implementation notes:
- *
- * The "cache" and "default registry" are different concepts and can represent
- * different sets of plugins. For various reasons, at init time, the cache is
- * stored in the default registry, and plugins not relevant to the current
- * process are marked with the GST_PLUGIN_FLAG_CACHED bit. These plugins are
- * removed at the end of intitialization.
- *
- */
-
 /* Element signals and args */
 enum
 {
@@ -314,7 +319,7 @@ gst_registry_add_plugin (GstRegistry * registry, GstPlugin * plugin)
 
   GST_OBJECT_LOCK (registry);
   existing_plugin = gst_registry_lookup_locked (registry, plugin->filename);
-  if (existing_plugin) {
+  if (G_UNLIKELY (existing_plugin)) {
     GST_DEBUG_OBJECT (registry,
         "Replacing existing plugin %p with new plugin %p for filename \"%s\"",
         existing_plugin, plugin, GST_STR_NULL (plugin->filename));
@@ -385,7 +390,7 @@ gst_registry_add_feature (GstRegistry * registry, GstPluginFeature * feature)
   GST_OBJECT_LOCK (registry);
   existing_feature = gst_registry_lookup_feature_locked (registry,
       feature->name);
-  if (existing_feature) {
+  if (G_UNLIKELY (existing_feature)) {
     GST_DEBUG_OBJECT (registry, "Replacing existing feature %p (%s)",
         existing_feature, feature->name);
     registry->features = g_list_remove (registry->features, existing_feature);
@@ -461,7 +466,7 @@ gst_registry_plugin_filter (GstRegistry * registry,
   list = gst_filter_run (registry->plugins, (GstFilterFunc) filter, first,
       user_data);
   for (g = list; g; g = g->next) {
-    gst_object_ref (GST_PLUGIN (g->data));
+    gst_object_ref (GST_PLUGIN_CAST (g->data));
   }
   GST_OBJECT_UNLOCK (registry);
 
@@ -497,7 +502,7 @@ gst_registry_feature_filter (GstRegistry * registry,
   list = gst_filter_run (registry->features, (GstFilterFunc) filter, first,
       user_data);
   for (g = list; g; g = g->next) {
-    gst_object_ref (GST_PLUGIN_FEATURE (g->data));
+    gst_object_ref (GST_PLUGIN_FEATURE_CAST (g->data));
   }
   GST_OBJECT_UNLOCK (registry);
 
@@ -529,7 +534,7 @@ gst_registry_find_plugin (GstRegistry * registry, const gchar * name)
   walk = gst_registry_plugin_filter (registry,
       (GstPluginFilter) gst_plugin_name_filter, TRUE, (gpointer) name);
   if (walk) {
-    result = GST_PLUGIN (walk->data);
+    result = GST_PLUGIN_CAST (walk->data);
 
     gst_object_ref (result);
     gst_plugin_list_free (walk);
@@ -571,7 +576,7 @@ gst_registry_find_feature (GstRegistry * registry, const gchar * name,
       TRUE, &data);
 
   if (walk) {
-    feature = GST_PLUGIN_FEATURE (walk->data);
+    feature = GST_PLUGIN_FEATURE_CAST (walk->data);
 
     gst_object_ref (feature);
     gst_plugin_feature_list_free (walk);
@@ -630,7 +635,7 @@ gst_registry_get_plugin_list (GstRegistry * registry)
   GST_OBJECT_LOCK (registry);
   list = g_list_copy (registry->plugins);
   for (g = list; g; g = g->next) {
-    gst_object_ref (GST_PLUGIN (g->data));
+    gst_object_ref (GST_PLUGIN_CAST (g->data));
   }
   GST_OBJECT_UNLOCK (registry);
 
@@ -643,11 +648,11 @@ gst_registry_lookup_feature_locked (GstRegistry * registry, const char *name)
   GList *g;
   GstPluginFeature *feature;
 
-  if (name == NULL)
+  if (G_UNLIKELY (name == NULL))
     return NULL;
 
   for (g = registry->features; g; g = g_list_next (g)) {
-    feature = GST_PLUGIN_FEATURE (g->data);
+    feature = GST_PLUGIN_FEATURE_CAST (g->data);
     if (feature->name && strcmp (name, feature->name) == 0) {
       return feature;
     }
@@ -692,12 +697,12 @@ gst_registry_lookup_locked (GstRegistry * registry, const char *filename)
   GstPlugin *plugin;
   gchar *basename;
 
-  if (filename == NULL)
+  if (G_UNLIKELY (filename == NULL))
     return NULL;
 
   basename = g_path_get_basename (filename);
   for (g = registry->plugins; g; g = g_list_next (g)) {
-    plugin = GST_PLUGIN (g->data);
+    plugin = GST_PLUGIN_CAST (g->data);
     if (plugin->basename && strcmp (basename, plugin->basename) == 0) {
       g_free (basename);
       return plugin;
@@ -736,7 +741,7 @@ gst_registry_lookup (GstRegistry * registry, const char *filename)
   return plugin;
 }
 
-static void
+static gboolean
 gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
     int level)
 {
@@ -745,10 +750,11 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
   gchar *filename;
   GstPlugin *plugin;
   GstPlugin *newplugin;
+  gboolean changed = FALSE;
 
   dir = g_dir_open (path, 0, NULL);
   if (!dir)
-    return;
+    return FALSE;
 
   while ((dirent = g_dir_read_name (dir))) {
     filename = g_strjoin ("/", path, dirent, NULL);
@@ -758,7 +764,7 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
     if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
       if (level > 0) {
         GST_LOG_OBJECT (registry, "found directory, recursing");
-        gst_registry_scan_path_level (registry, filename, level - 1);
+        changed |= gst_registry_scan_path_level (registry, filename, level - 1);
       } else {
         GST_LOG_OBJECT (registry,
             "found directory, but recursion level is too deep");
@@ -819,6 +825,7 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
           newplugin->registered = TRUE;
           gst_object_unref (newplugin);
         }
+        changed = TRUE;
       }
       gst_object_unref (plugin);
 
@@ -828,6 +835,7 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
       if (newplugin) {
         newplugin->registered = TRUE;
         gst_object_unref (newplugin);
+        changed = TRUE;
       }
     }
 
@@ -835,6 +843,10 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
   }
 
   g_dir_close (dir);
+
+  GST_DEBUG_OBJECT (registry, "registry changed? %d", changed);
+
+  return changed;
 }
 
 /**
@@ -845,15 +857,23 @@ gst_registry_scan_path_level (GstRegistry * registry, const gchar * path,
  * Add the given path to the registry. The syntax of the
  * path is specific to the registry. If the path has already been
  * added, do nothing.
+ *
+ * Returns: %TRUE if registry changed
  */
-void
+gboolean
 gst_registry_scan_path (GstRegistry * registry, const gchar * path)
 {
-  g_return_if_fail (GST_IS_REGISTRY (registry));
-  g_return_if_fail (path != NULL);
+  gboolean changed;
+
+  g_return_val_if_fail (GST_IS_REGISTRY (registry), FALSE);
+  g_return_val_if_fail (path != NULL, FALSE);
 
   GST_DEBUG_OBJECT (registry, "scanning path %s", path);
-  gst_registry_scan_path_level (registry, path, 10);
+  changed = gst_registry_scan_path_level (registry, path, 10);
+
+  GST_DEBUG_OBJECT (registry, "registry changed? %d", changed);
+
+  return changed;
 }
 
 void
