@@ -99,7 +99,7 @@ gst_util_set_value_from_string (GValue * value, const gchar * value_str)
 
   switch (G_VALUE_TYPE (value)) {
     case G_TYPE_STRING:
-      g_value_set_string (value, g_strdup (value_str));
+      g_value_set_string (value, value_str);
       break;
     case G_TYPE_ENUM:
     case G_TYPE_INT:{
@@ -1482,13 +1482,8 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "no link possible from %s to %s:%s",
         GST_ELEMENT_NAME (src), GST_DEBUG_PAD_NAME (destpad));
     gst_object_unref (destpad);
-    if (srcpad)
-      gst_object_unref (srcpad);
     return FALSE;
   } else {
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
     if (destpad)
       gst_object_unref (destpad);
     destpad = NULL;
@@ -1598,8 +1593,9 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
       return TRUE;
     } else {
       GST_INFO ("Could not link elements");
+      gst_element_set_state (capsfilter, GST_STATE_NULL);
+      /* this will unlink and unref as appropriate */
       gst_bin_remove (GST_BIN (GST_OBJECT_PARENT (capsfilter)), capsfilter);
-      /* will unref and unlink as appropriate */
       return FALSE;
     }
   } else {
@@ -1617,6 +1613,9 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
  * existing pads that aren't linked yet. It will request new pads if necessary.
  * If multiple links are possible, only one is established.
  *
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
+ *
  * Returns: TRUE if the elements could be linked, FALSE otherwise.
  */
 gboolean
@@ -1632,6 +1631,8 @@ gst_element_link (GstElement * src, GstElement * dest)
  * @...: the NULL-terminated list of elements to link in order.
  *
  * Chain together a series of elements. Uses gst_element_link().
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
  *
  * Returns: TRUE on success, FALSE otherwise.
  */
@@ -1670,6 +1671,9 @@ gst_element_link_many (GstElement * element_1, GstElement * element_2, ...)
  * existing pads that aren't linked yet. It will request new pads if necessary.
  * If multiple links are possible, only one is established.
  *
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
+ *
  * Returns: TRUE if the pads could be linked, FALSE otherwise.
  */
 gboolean
@@ -1811,6 +1815,7 @@ gst_element_unlink (GstElement * src, GstElement * dest)
         break;
     }
   }
+  gst_iterator_free (pads);
 }
 
 /**
@@ -1916,6 +1921,40 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
   return ret;
 }
 
+/**
+ * gst_element_seek_simple
+ * @element: a #GstElement to seek on
+ * @format: a #GstFormat to execute the seek in, such as #GST_FORMAT_TIME
+ * @seek_flags: seek options
+ * @seek_pos: position to seek to (relative to the start); if you are doing
+ *            a seek in #GST_FORMAT_TIME this value is in nanoseconds -
+ *            multiply with #GST_SECOND to convert seconds to nanoseconds or
+ *            with #GST_MSECOND to convert milliseconds to nanoseconds.
+ *
+ * Simple API to perform a seek on the given element, meaning it just seeks
+ * to the given position relative to the start of the stream. For more complex
+ * operations like segment seeks (e.g. for looping) or changing the playback
+ * rate or seeking relative to the current position or seeking relative to
+ * the end of the stream you should use gst_element_seek ().
+ *
+ * Note that seeking is usually only possible in PAUSED or PLAYING state.
+ *
+ * Returns: TRUE if the seek operation succeeded (the seek
+ *          might not always be executed instantly though)
+ *
+ * Since: 0.10.7
+ */
+gboolean
+gst_element_seek_simple (GstElement * element, GstFormat format,
+    GstSeekFlags seek_flags, gint64 seek_pos)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (seek_pos >= 0, FALSE);
+
+  return gst_element_seek (element, 1.0, format, seek_flags,
+      GST_SEEK_TYPE_SET, seek_pos, GST_SEEK_TYPE_NONE, 0);
+}
+
 /**
  * gst_pad_can_link:
  * @srcpad: the source #GstPad to link.
@@ -2011,6 +2050,7 @@ gst_pad_get_fixed_caps_func (GstPad * pad)
 
   g_return_val_if_fail (GST_IS_PAD (pad), NULL);
 
+  GST_OBJECT_LOCK (pad);
   if (GST_PAD_CAPS (pad)) {
     result = GST_PAD_CAPS (pad);
 
@@ -2035,6 +2075,8 @@ gst_pad_get_fixed_caps_func (GstPad * pad)
   result = gst_caps_new_empty ();
 
 done:
+  GST_OBJECT_UNLOCK (pad);
+
   return result;
 }
 
@@ -2386,18 +2428,40 @@ gst_pad_proxy_getcaps (GstPad * pad)
   if (element == NULL)
     return NULL;
 
-  iter = gst_element_iterate_pads (element);
-
+  /* value to hold the return, by default it holds ANY, the ref is taken by
+   * the GValue. */
   g_value_init (&ret, G_TYPE_POINTER);
   g_value_set_pointer (&ret, gst_caps_new_any ());
 
-  res = gst_iterator_fold (iter, (GstIteratorFoldFunction) intersect_caps_func,
-      &ret, pad);
+  iter = gst_element_iterate_pads (element);
+  while (1) {
+    res =
+        gst_iterator_fold (iter, (GstIteratorFoldFunction) intersect_caps_func,
+        &ret, pad);
+    switch (res) {
+      case GST_ITERATOR_RESYNC:
+        /* unref any value stored */
+        if ((caps = g_value_get_pointer (&ret)))
+          gst_caps_unref (caps);
+        /* need to reset the result again to ANY */
+        g_value_set_pointer (&ret, gst_caps_new_any ());
+        gst_iterator_resync (iter);
+        break;
+      case GST_ITERATOR_DONE:
+        /* all pads iterated, return collected value */
+        goto done;
+      default:
+        /* iterator returned _ERROR or premature end with _OK,
+         * mark an error and exit */
+        if ((caps = g_value_get_pointer (&ret)))
+          gst_caps_unref (caps);
+        g_value_set_pointer (&ret, NULL);
+        goto error;
+    }
+  }
+done:
   gst_iterator_free (iter);
 
-  if (res != GST_ITERATOR_DONE)
-    goto pads_changed;
-
   gst_object_unref (element);
 
   caps = g_value_get_pointer (&ret);
@@ -2409,10 +2473,11 @@ gst_pad_proxy_getcaps (GstPad * pad)
   return intersected;
 
   /* ERRORS */
-pads_changed:
+error:
   {
-    g_warning ("Pad list changed during capsnego for element %s",
+    g_warning ("Pad list returned error on element %s",
         GST_ELEMENT_NAME (element));
+    gst_iterator_free (iter);
     gst_object_unref (element);
     return NULL;
   }
@@ -2749,19 +2814,10 @@ gulong
 gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2771,10 +2827,6 @@ gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_PAD_DO_BUFFER_SIGNALS (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2793,19 +2845,10 @@ gulong
 gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::event", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2813,10 +2856,6 @@ gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2835,19 +2874,10 @@ gulong
 gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::buffer", handler, data);
   GST_PAD_DO_BUFFER_SIGNALS (pad)++;
@@ -2855,10 +2885,6 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2872,19 +2898,9 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 void
 gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2895,9 +2911,6 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
       GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2910,29 +2923,15 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_EVENT_SIGNALS (pad)--;
   GST_DEBUG ("removed event probe from pad %s:%s, now %d event probes",
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
-
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2945,19 +2944,9 @@ gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2965,9 +2954,6 @@ gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
