@@ -26,16 +26,16 @@
  *
  * #GstBaseSink is the base class for sink elements in GStreamer, such as
  * xvimagesink or filesink. It is a layer on top of #GstElement that provides a
- * simplified interface to plugin writers. #GstBaseSink handles many details 
- * for you, for example: preroll, clock synchronization, state changes, 
- * activation in push or pull mode, and queries. 
- * 
- * In most cases, when writing sink elements, there is no need to implement 
- * class methods from #GstElement or to set functions on pads, because the 
+ * simplified interface to plugin writers. #GstBaseSink handles many details
+ * for you, for example: preroll, clock synchronization, state changes,
+ * activation in push or pull mode, and queries.
+ *
+ * In most cases, when writing sink elements, there is no need to implement
+ * class methods from #GstElement or to set functions on pads, because the
  * #GstBaseSink infrastructure should be sufficient.
  *
- * #GstBaseSink provides support for exactly one sink pad, which should be 
- * named "sink". A sink implementation (subclass of GstBaseSink) should 
+ * #GstBaseSink provides support for exactly one sink pad, which should be
+ * named "sink". A sink implementation (subclass of GstBaseSink) should
  * install a pad template in its base_init function, like so:
  * <programlisting>
  * static void
@@ -53,15 +53,15 @@
  * </programlisting>
  *
  * #GstBaseSink will handle the prerolling correctly. This means that it will
- * return GST_STATE_CHANGE_ASYNC from a state change to PAUSED until the first 
- * buffer arrives in this element. The base class will call the 
- * GstBaseSink::preroll vmethod with this preroll buffer and will then commit 
+ * return GST_STATE_CHANGE_ASYNC from a state change to PAUSED until the first
+ * buffer arrives in this element. The base class will call the
+ * GstBaseSink::preroll vmethod with this preroll buffer and will then commit
  * the state change to the next asynchronously pending state.
  *
- * When the element is set to PLAYING, #GstBaseSink will synchronise on the 
+ * When the element is set to PLAYING, #GstBaseSink will synchronise on the
  * clock using the times returned from ::get_times. If this function returns
  * #GST_CLOCK_TIME_NONE for the start time, no synchronisation will be done.
- * Synchronisation can be disabled entirely by setting the object "sync" 
+ * Synchronisation can be disabled entirely by setting the object "sync"
  * property to FALSE.
  *
  * After synchronisation the virtual method #GstBaseSink::render will be called.
@@ -70,61 +70,61 @@
  * Since 0.10.3 subclasses that synchronise on the clock in the ::render method
  * are supported as well. These classes typically receive a buffer in the render
  * method and can then potentially block on the clock while rendering. A typical
- * example is an audiosink. 
+ * example is an audiosink.
  *
- * Upon receiving the EOS event in the PLAYING state, #GstBaseSink will wait 
- * for the clock to reach the time indicated by the stop time of the last 
- * ::get_times call before posting an EOS message. When the element receives 
- * EOS in PAUSED, preroll completes, the event is queued and an EOS message is 
+ * Upon receiving the EOS event in the PLAYING state, #GstBaseSink will wait
+ * for the clock to reach the time indicated by the stop time of the last
+ * ::get_times call before posting an EOS message. When the element receives
+ * EOS in PAUSED, preroll completes, the event is queued and an EOS message is
  * posted when going to PLAYING.
- * 
+ *
  * #GstBaseSink will internally use the GST_EVENT_NEW_SEGMENT events to schedule
  * synchronisation and clipping of buffers. Buffers that fall completely outside
- * of the current segment are dropped. Buffers that fall partially in the 
- * segment are rendered (and prerolled). Subclasses should do any subbuffer 
+ * of the current segment are dropped. Buffers that fall partially in the
+ * segment are rendered (and prerolled). Subclasses should do any subbuffer
  * clipping themselves when needed.
- * 
- * #GstBaseSink will by default report the current playback position in 
- * GST_FORMAT_TIME based on the current clock time and segment information. 
- * If no clock has been set on the element, the query will be forwarded 
+ *
+ * #GstBaseSink will by default report the current playback position in
+ * GST_FORMAT_TIME based on the current clock time and segment information.
+ * If no clock has been set on the element, the query will be forwarded
  * upstream.
  *
- * The ::set_caps function will be called when the subclass should configure 
+ * The ::set_caps function will be called when the subclass should configure
  * itself to process a specific media type.
- * 
- * The ::start and ::stop virtual methods will be called when resources should 
- * be allocated. Any ::preroll, ::render  and ::set_caps function will be 
- * called between the ::start and ::stop calls. 
  *
- * The ::event virtual method will be called when an event is received by 
+ * The ::start and ::stop virtual methods will be called when resources should
+ * be allocated. Any ::preroll, ::render  and ::set_caps function will be
+ * called between the ::start and ::stop calls.
+ *
+ * The ::event virtual method will be called when an event is received by
  * #GstBaseSink. Normally this method should only be overriden by very specific
- * elements (such as file sinks) which need to handle the newsegment event 
+ * elements (such as file sinks) which need to handle the newsegment event
  * specially.
- * 
- * #GstBaseSink provides an overridable ::buffer_alloc function that can be 
+ *
+ * #GstBaseSink provides an overridable ::buffer_alloc function that can be
  * used by sinks that want to do reverse negotiation or to provide
  * custom buffers (hardware buffers for example) to upstream elements.
  *
  * The ::unlock method is called when the elements should unblock any blocking
  * operations they perform in the ::render method. This is mostly useful when
- * the ::render method performs a blocking write on a file descriptor, for 
+ * the ::render method performs a blocking write on a file descriptor, for
  * example.
  *
- * The max-lateness property affects how the sink deals with buffers that 
- * arrive too late in the sink. A buffer arrives too late in the sink when 
- * the presentation time (as a combination of the last segment, buffer 
- * timestamp and element base_time) plus the duration is before the current 
- * time of the clock. 
+ * The max-lateness property affects how the sink deals with buffers that
+ * arrive too late in the sink. A buffer arrives too late in the sink when
+ * the presentation time (as a combination of the last segment, buffer
+ * timestamp and element base_time) plus the duration is before the current
+ * time of the clock.
  * If the frame is later than max-lateness, the sink will drop the buffer
- * without calling the render method. 
- * This feature is disabled if sync is disabled, the ::get-times method does 
- * not return a valid start time or max-lateness is set to -1 (the default). 
- * Subclasses can use gst_base_sink_set_max_lateness() to configure the 
+ * without calling the render method.
+ * This feature is disabled if sync is disabled, the ::get-times method does
+ * not return a valid start time or max-lateness is set to -1 (the default).
+ * Subclasses can use gst_base_sink_set_max_lateness() to configure the
  * max-lateness value.
  *
- * The qos property will enable the quality-of-service features of the basesink 
- * which gather statistics about the real-time performance of the clock 
- * synchronisation. For each dropped buffer it will also send a QoS message 
+ * The qos property will enable the quality-of-service features of the basesink
+ * which gather statistics about the real-time performance of the clock
+ * synchronisation. For each dropped buffer it will also send a QoS message
  * upstream.
  *
  * Last reviewed on 2006-03-31 (0.10.5)
@@ -145,7 +145,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_base_sink_debug);
 #define GST_BASE_SINK_GET_PRIVATE(obj)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_BASE_SINK, GstBaseSinkPrivate))
 
-/* FIXME, some stuff in ABI.data and other in Private... 
+/* FIXME, some stuff in ABI.data and other in Private...
  * Make up your mind please.
  */
 struct _GstBaseSinkPrivate
@@ -161,6 +161,9 @@ struct _GstBaseSinkPrivate
   GstClockTime current_rstop;
   GstClockTimeDiff current_jitter;
 
+  /* EOS sync time in running time */
+  GstClockTime eos_rtime;
+
   /* last buffer that arrived in time, running time */
   GstClockTime last_in_time;
   /* when the last buffer left the sink, running time */
@@ -171,9 +174,9 @@ struct _GstBaseSinkPrivate
   GstClockTime avg_duration;
   gdouble avg_rate;
 
-  /* these are done on system time. avg_jitter and avg_render are 
+  /* these are done on system time. avg_jitter and avg_render are
    * compared to eachother to see if the rendering time takes a
-   * huge amount of the processing, If so we are flooded with 
+   * huge amount of the processing, If so we are flooded with
    * buffers. */
   GstClockTime last_left_systime;
   GstClockTime avg_jitter;
@@ -187,8 +190,12 @@ struct _GstBaseSinkPrivate
 
 #define DO_RUNNING_AVG(avg,val,size) (((val) + ((size)-1) * (avg)) / (size))
 
+/* generic running average, this has a neutral window size */
 #define UPDATE_RUNNING_AVG(avg,val)   DO_RUNNING_AVG(avg,val,8)
 
+/* the windows for these running averages are experimentally obtained.
+ * possitive values get averaged more while negative values use a small
+ * window so we can react faster to badness. */
 #define UPDATE_RUNNING_AVG_P(avg,val) DO_RUNNING_AVG(avg,val,16)
 #define UPDATE_RUNNING_AVG_N(avg,val) DO_RUNNING_AVG(avg,val,4)
 
@@ -204,8 +211,8 @@ enum
 #define DEFAULT_CAN_ACTIVATE_PULL FALSE /* fixme: enable me */
 #define DEFAULT_CAN_ACTIVATE_PUSH TRUE
 
-#define DEFAULT_PREROLL_QUEUE_LEN 	0
-#define DEFAULT_SYNC 			TRUE
+#define DEFAULT_PREROLL_QUEUE_LEN	0
+#define DEFAULT_SYNC			TRUE
 #define DEFAULT_MAX_LATENESS		-1
 #define DEFAULT_QOS			FALSE
 
@@ -230,7 +237,7 @@ gst_base_sink_get_type (void)
 {
   static GType base_sink_type = 0;
 
-  if (!base_sink_type) {
+  if (G_UNLIKELY (base_sink_type == 0)) {
     static const GTypeInfo base_sink_info = {
       sizeof (GstBaseSinkClass),
       (GBaseInitFunc) gst_base_sink_base_init,
@@ -295,8 +302,8 @@ gst_base_sink_class_init (GstBaseSinkClass * klass)
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
+  gobject_class = G_OBJECT_CLASS (klass);
+  gstelement_class = GST_ELEMENT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GstBaseSinkPrivate));
 
@@ -308,24 +315,22 @@ gst_base_sink_class_init (GstBaseSinkClass * klass)
 
   /* FIXME, this next value should be configured using an event from the
    * upstream element, ie, the BUFFER_SIZE event. */
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
-      PROP_PREROLL_QUEUE_LEN,
+  g_object_class_install_property (gobject_class, PROP_PREROLL_QUEUE_LEN,
       g_param_spec_uint ("preroll-queue-len", "Preroll queue length",
           "Number of buffers to queue during preroll", 0, G_MAXUINT,
           DEFAULT_PREROLL_QUEUE_LEN, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SYNC,
+  g_object_class_install_property (gobject_class, PROP_SYNC,
       g_param_spec_boolean ("sync", "Sync", "Sync on the clock", DEFAULT_SYNC,
           G_PARAM_READWRITE));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
-      PROP_MAX_LATENESS,
+  g_object_class_install_property (gobject_class, PROP_MAX_LATENESS,
       g_param_spec_int64 ("max-lateness", "Max Lateness",
           "Maximum number of nanoseconds that a buffer can be late before it "
           "is dropped (-1 unlimited)", -1, G_MAXINT64, DEFAULT_MAX_LATENESS,
           G_PARAM_READWRITE));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_QOS,
+  g_object_class_install_property (gobject_class, PROP_QOS,
       g_param_spec_boolean ("qos", "Qos", "Generate QoS events upstream",
           DEFAULT_QOS, G_PARAM_READWRITE));
 
@@ -556,7 +561,7 @@ gst_base_sink_get_max_lateness (GstBaseSink * sink)
 /**
  * gst_base_sink_set_qos_enabled:
  * @sink: the sink
- * @qos: the new qos value.
+ * @enabled: the new qos value.
  *
  * Configures @sink to send QoS events upstream.
  *
@@ -693,7 +698,7 @@ gst_base_sink_configure_segment (GstBaseSink * basesink, GstPad * pad,
     GstEvent * event, GstSegment * segment)
 {
   gboolean update;
-  gdouble rate;
+  gdouble rate, arate;
   GstFormat format;
   gint64 start;
   gint64 stop;
@@ -701,29 +706,31 @@ gst_base_sink_configure_segment (GstBaseSink * basesink, GstPad * pad,
 
   /* the newsegment event is needed to bring the buffer timestamps to the
    * stream time and to drop samples outside of the playback segment. */
-  gst_event_parse_new_segment (event, &update, &rate, &format,
+  gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
       &start, &stop, &time);
 
+  /* The segment is protected with both the STREAM_LOCK and the OBJECT_LOCK.
+   * We protect with the OBJECT_LOCK so that we can use the values to
+   * safely answer a POSITION query. */
   GST_OBJECT_LOCK (basesink);
-
-  if (segment->format != format)
-    gst_segment_init (segment, format);
-
-  gst_segment_set_newsegment (segment, update, rate, format, start, stop, time);
+  gst_segment_set_newsegment_full (segment, update, rate, arate, format, start,
+      stop, time);
 
   if (format == GST_FORMAT_TIME) {
     GST_DEBUG_OBJECT (basesink,
-        "configured NEWSEGMENT update %d, rate %lf, format GST_FORMAT_TIME, "
+        "configured NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+        "format GST_FORMAT_TIME, "
         "%" GST_TIME_FORMAT " -- %" GST_TIME_FORMAT
         ", time %" GST_TIME_FORMAT ", accum %" GST_TIME_FORMAT,
-        update, rate, GST_TIME_ARGS (segment->start),
+        update, rate, arate, GST_TIME_ARGS (segment->start),
         GST_TIME_ARGS (segment->stop), GST_TIME_ARGS (segment->time),
         GST_TIME_ARGS (segment->accum));
   } else {
     GST_DEBUG_OBJECT (basesink,
-        "configured NEWSEGMENT update %d, rate %lf, format %d, "
+        "configured NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+        "format %d, "
         "%" G_GINT64_FORMAT " -- %" G_GINT64_FORMAT ", time %"
-        G_GINT64_FORMAT ", accum %" G_GINT64_FORMAT, update, rate,
+        G_GINT64_FORMAT ", accum %" G_GINT64_FORMAT, update, rate, arate,
         segment->format, segment->start, segment->stop, segment->time,
         segment->accum);
   }
@@ -868,7 +875,10 @@ gst_base_sink_get_sync_times (GstBaseSink * basesink, GstMiniObject * obj,
         /* EOS event needs syncing */
       case GST_EVENT_EOS:
         sstart = sstop = basesink->priv->current_sstop;
-        rstart = rstop = basesink->priv->current_rstop;
+        rstart = rstop = basesink->priv->eos_rtime;
+        *do_sync = rstart != -1;
+        GST_DEBUG_OBJECT (basesink, "sync times for EOS %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (rstart));
         goto done;
         /* other events do not need syncing */
         /* FIXME, maybe NEWSEGMENT might need synchronisation
@@ -898,7 +908,8 @@ gst_base_sink_get_sync_times (GstBaseSink * basesink, GstMiniObject * obj,
   }
 
   GST_DEBUG_OBJECT (basesink, "got times start: %" GST_TIME_FORMAT
-      ", stop: %" GST_TIME_FORMAT, GST_TIME_ARGS (start), GST_TIME_ARGS (stop));
+      ", stop: %" GST_TIME_FORMAT ", do_sync %d", GST_TIME_ARGS (start),
+      GST_TIME_ARGS (stop), *do_sync);
 
   /* collect segment and format for code clarity */
   segment = &basesink->segment;
@@ -922,6 +933,9 @@ gst_base_sink_get_sync_times (GstBaseSink * basesink, GstMiniObject * obj,
         GST_TIME_ARGS (cstop));
   }
 
+  /* set last stop position */
+  gst_segment_set_last_stop (segment, GST_FORMAT_TIME, cstop);
+
 do_times:
   /* this can produce wrong values if we accumulated non-TIME segments. If this happens,
    * upstream is behaving very badly */
@@ -1069,6 +1083,9 @@ gst_base_sink_do_sync (GstBaseSink * basesink, GstPad * pad,
   /* store timing info for current object */
   basesink->priv->current_rstart = rstart;
   basesink->priv->current_rstop = (rstop != -1 ? rstop : rstart);
+  /* save sync time for eos when the previous object needed sync */
+  basesink->priv->eos_rtime = (do_sync ? basesink->priv->current_rstop : -1);
+
   /* lock because we read this when answering the POSITION 
    * query. */
   GST_OBJECT_LOCK (basesink);
@@ -1341,12 +1358,24 @@ gst_base_sink_is_too_late (GstBaseSink * basesink, GstMiniObject * obj,
   if (G_UNLIKELY (start == -1))
     goto no_timestamp;
 
+  /* we can add a valid stop time */
+  if (stop != -1)
+    max_lateness += stop;
+  else
+    max_lateness += start;
+
   /* if the jitter bigger than duration and lateness we are too late */
-  if (G_LIKELY (stop != -1)) {
-    if ((late = start + jitter > stop + max_lateness)) {
-      if (priv->last_in_time && start - priv->last_in_time > GST_SECOND) {
-        late = FALSE;
-      }
+  if ((late = start + jitter > max_lateness)) {
+    GST_DEBUG_OBJECT (basesink, "buffer is too late %" GST_TIME_FORMAT
+        " > %" GST_TIME_FORMAT, GST_TIME_ARGS (start + jitter),
+        GST_TIME_ARGS (max_lateness));
+    /* !!emergency!!, if we did not receive anything valid for more than a 
+     * second, render it anyway so the user sees something */
+    if (priv->last_in_time && start - priv->last_in_time > GST_SECOND) {
+      late = FALSE;
+      GST_DEBUG_OBJECT (basesink,
+          "**emergency** last buffer at %" GST_TIME_FORMAT " > GST_SECOND",
+          GST_TIME_ARGS (priv->last_in_time));
     }
   }
 
@@ -1401,8 +1430,6 @@ gst_base_sink_do_render_stats (GstBaseSink * basesink, gboolean start)
     else
       priv->avg_render = UPDATE_RUNNING_AVG (priv->avg_render, elapsed);
 
-    priv->rendered++;
-
     GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, basesink,
         "avg_render: %" GST_TIME_FORMAT, GST_TIME_ARGS (priv->avg_render));
   }
@@ -1453,6 +1480,8 @@ gst_base_sink_render_object (GstBaseSink * basesink, GstPad * pad,
 
       ret = bclass->render (basesink, GST_BUFFER_CAST (obj));
 
+      priv->rendered++;
+
       if (do_qos)
         gst_base_sink_do_render_stats (basesink, FALSE);
     }
@@ -1546,7 +1575,6 @@ gst_base_sink_preroll_object (GstBaseSink * basesink, GstPad * pad,
     if (bclass->preroll)
       if ((ret = bclass->preroll (basesink, buf)) != GST_FLOW_OK)
         goto preroll_failed;
-  } else {
   }
 
   /* commit state */
@@ -1691,7 +1719,8 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
 
   bclass = GST_BASE_SINK_GET_CLASS (basesink);
 
-  GST_DEBUG_OBJECT (basesink, "event %p", event);
+  GST_DEBUG_OBJECT (basesink, "event %p (%s)", event,
+      GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
@@ -1851,11 +1880,10 @@ gst_base_sink_chain_unlocked (GstBaseSink * basesink, GstPad * pad,
     }
 
     basesink->have_newsegment = TRUE;
-    /* this means this sink will not use segment info for sync */
-    clip_segment->start = -1;
+    /* this means this sink will assume timestamps start from 0 */
+    clip_segment->start = 0;
     clip_segment->stop = -1;
-    /* and we set the rendering segment to invalid as well */
-    basesink->segment.start = -1;
+    basesink->segment.start = 0;
     basesink->segment.stop = -1;
   }
 
@@ -1955,6 +1983,8 @@ gst_base_sink_loop (GstPad * pad)
   if (G_UNLIKELY (buf == NULL))
     goto no_buffer;
 
+  basesink->offset += GST_BUFFER_SIZE (buf);
+
   GST_PAD_PREROLL_LOCK (pad);
   result = gst_base_sink_chain_unlocked (basesink, pad, buf);
   GST_PAD_PREROLL_UNLOCK (pad);
@@ -1996,23 +2026,27 @@ static gboolean
 gst_base_sink_set_flushing (GstBaseSink * basesink, GstPad * pad,
     gboolean flushing)
 {
-  GST_PAD_PREROLL_LOCK (pad);
-  basesink->flushing = flushing;
+
   if (flushing) {
     GstBaseSinkClass *bclass;
 
     bclass = GST_BASE_SINK_GET_CLASS (basesink);
 
+    /* unlock any subclasses, we need to do this before grabbing the
+     * PREROLL_LOCK since we hold this lock before going into ::render. */
+    if (bclass->unlock)
+      bclass->unlock (basesink);
+  }
+
+  GST_PAD_PREROLL_LOCK (pad);
+  basesink->flushing = flushing;
+  if (flushing) {
     /* step 1, unblock clock sync (if any) or any other blocking thing */
     basesink->need_preroll = TRUE;
     if (basesink->clock_id) {
       gst_clock_id_unschedule (basesink->clock_id);
     }
 
-    /* unlock any subclasses */
-    if (bclass->unlock)
-      bclass->unlock (basesink);
-
     /* flush out the data thread if it's locked in finish_preroll */
     GST_DEBUG_OBJECT (basesink,
         "flushing out data thread, need preroll to TRUE");
@@ -2196,7 +2230,9 @@ gst_base_sink_peer_query (GstBaseSink * sink, GstQuery * query)
   return res;
 }
 
-/* get the position of the last seen object */
+/* get the end position of the last seen object, this is used
+ * for EOS and for making sure that we don't report a position we
+ * have not reached yet. */
 static gboolean
 gst_base_sink_get_position_last (GstBaseSink * basesink, gint64 * cur)
 {
@@ -2230,30 +2266,38 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
   gboolean res = FALSE;
 
   switch (format) {
+      /* we can answer time format */
     case GST_FORMAT_TIME:
     {
       GstClockTime now, base;
       gint64 time, accum;
-      gdouble abs_rate;
+      gdouble rate;
       gint64 last;
 
-      /* we can answer time format */
       GST_OBJECT_LOCK (basesink);
 
-      /* can only give answer if not EOS */
+      /* can only give answer based on the clock if not EOS */
       if (G_UNLIKELY (basesink->eos))
         goto in_eos;
 
+      /* in PAUSE we cannot read from the clock so we
+       * report time based on the last seen timestamp. */
       if (GST_STATE (basesink) == GST_STATE_PAUSED)
         goto in_pause;
 
-      /* We get position from clock only in PLAYING */
+      /* We get position from clock only in PLAYING, we checked
+       * the PAUSED case above, so this is check is to test 
+       * READY and NULL, where the position is always 0 */
       if (GST_STATE (basesink) != GST_STATE_PLAYING)
         goto wrong_state;
 
+      /* we need to sync on the clock. */
+      if (basesink->sync == FALSE)
+        goto no_sync;
+
       /* and we need a clock */
       if (G_UNLIKELY ((clock = GST_ELEMENT_CLOCK (basesink)) == NULL))
-        goto no_clock;
+        goto no_sync;
 
       /* collect all data we need holding the lock */
       if (GST_CLOCK_TIME_IS_VALID (basesink->segment.time))
@@ -2263,20 +2307,26 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
 
       base = GST_ELEMENT_CAST (basesink)->base_time;
       accum = basesink->segment.accum;
-      abs_rate = basesink->segment.abs_rate;
+      rate = basesink->segment.rate * basesink->segment.applied_rate;
       gst_base_sink_get_position_last (basesink, &last);
 
       gst_object_ref (clock);
       /* need to release the object lock before we can get the time, 
-       * a clock might take the LOCK of the provider. */
+       * a clock might take the LOCK of the provider, which could be
+       * a basesink subclass. */
       GST_OBJECT_UNLOCK (basesink);
 
       now = gst_clock_get_time (clock);
+      /* subtract base time and accumulated time from the clock time. 
+       * Make sure we don't go negative. This is the current time in
+       * the segment which we need to scale with the combined 
+       * rate and applied rate. */
       base += accum;
       base = MIN (now, base);
-      /* never report more than last seen position */
-      *cur = gst_guint64_to_gdouble (now - base) * abs_rate + time;
+      /* for negative rate this will count back from the segment time */
+      *cur = time + gst_guint64_to_gdouble (now - base) * rate;
 
+      /* never report more than last seen position */
       if (last != -1)
         *cur = MIN (last, *cur);
 
@@ -2291,7 +2341,8 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
           GST_TIME_ARGS (accum), GST_TIME_ARGS (time));
     }
     default:
-      /* cannot answer other than TIME */
+      /* cannot answer other than TIME, we return FALSE, which will
+       * send the query upstream. */
       break;
   }
 
@@ -2315,15 +2366,20 @@ in_pause:
   }
 wrong_state:
   {
+    /* in NULL or READY we always return 0 */
     res = TRUE;
     *cur = 0;
     GST_OBJECT_UNLOCK (basesink);
     goto done;
   }
-no_clock:
+no_sync:
   {
+    /* report last seen timestamp if any, else return FALSE so
+     * that upstream can answer */
+    if ((*cur = basesink->priv->current_sstart) != -1)
+      res = TRUE;
     GST_OBJECT_UNLOCK (basesink);
-    return FALSE;
+    return res;
   }
 }
 
@@ -2411,6 +2467,7 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
       basesink->playing_async = TRUE;
       basesink->priv->current_sstart = 0;
       basesink->priv->current_sstop = 0;
+      basesink->priv->eos_rtime = -1;
       basesink->eos = FALSE;
       gst_base_sink_reset_qos (basesink);
       ret = GST_STATE_CHANGE_ASYNC;
@@ -2454,15 +2511,18 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED");
+
+      /* we need to call ::unlock before locking PREROLL_LOCK
+       * since we lock it before going into ::render */
+      if (bclass->unlock)
+        bclass->unlock (basesink);
+
       GST_PAD_PREROLL_LOCK (basesink->sinkpad);
       basesink->need_preroll = TRUE;
       if (basesink->clock_id) {
         gst_clock_id_unschedule (basesink->clock_id);
       }
 
-      if (bclass->unlock)
-        bclass->unlock (basesink);
-
       /* if we don't have a preroll buffer we need to wait for a preroll and
        * return ASYNC. */
       if (gst_base_sink_is_prerolled (basesink)) {
@@ -2472,6 +2532,10 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
         basesink->playing_async = TRUE;
         ret = GST_STATE_CHANGE_ASYNC;
       }
+      GST_DEBUG_OBJECT (basesink, "rendered: %" G_GUINT64_FORMAT
+          ", dropped: %" G_GUINT64_FORMAT, basesink->priv->rendered,
+          basesink->priv->dropped);
+
       gst_base_sink_reset_qos (basesink);
       GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
       break;
