@@ -431,10 +431,11 @@ gst_base_transform_transform_caps (GstBaseTransform * trans,
 
     /* start with empty caps */
     ret = gst_caps_new_empty ();
+    GST_DEBUG_OBJECT (trans, "transform caps (direction = %d)", direction);
 
     if (gst_caps_is_any (caps)) {
       /* for any caps we still have to call the transform function */
-      GST_DEBUG_OBJECT (trans, "from ANY:");
+      GST_DEBUG_OBJECT (trans, "from: ANY");
       temp = klass->transform_caps (trans, direction, caps);
       GST_DEBUG_OBJECT (trans, "  to: %" GST_PTR_FORMAT, temp);
 
@@ -453,15 +454,25 @@ gst_base_transform_transform_caps (GstBaseTransform * trans,
         GST_DEBUG_OBJECT (trans, "  to[%d]: %" GST_PTR_FORMAT, i, temp);
 
         temp = gst_caps_make_writable (temp);
-        gst_caps_append (ret, temp);
+        /* FIXME: here we need to only append those structures, that are not yet
+         * in there
+         * gst_caps_append (ret, temp);
+         */
+        gst_caps_merge (ret, temp);
       }
+      GST_DEBUG_OBJECT (trans, "merged: (%d)", gst_caps_get_size (ret));
+      /* now simplify caps
+         gst_caps_do_simplify (ret);
+         GST_DEBUG_OBJECT (trans, "simplified: (%d)", gst_caps_get_size (ret));
+       */
     }
   } else {
     /* else use the identity transform */
     ret = gst_caps_ref (caps);
   }
 
-  GST_DEBUG_OBJECT (trans, "to:   %" GST_PTR_FORMAT, ret);
+  GST_DEBUG_OBJECT (trans, "to: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (ret),
+      ret);
 
   return ret;
 }
@@ -770,7 +781,8 @@ gst_base_transform_setcaps (GstPad * pad, GstCaps * caps)
   if (!peer_checked && otherpeer && !gst_pad_accept_caps (otherpeer, othercaps))
     goto peer_no_accept;
 
-  GST_DEBUG_OBJECT (trans, "got final caps %" GST_PTR_FORMAT, othercaps);
+  GST_DEBUG_OBJECT (trans, "Input caps were %" GST_PTR_FORMAT
+      ", and got final caps %" GST_PTR_FORMAT, caps, othercaps);
 
   trans->have_same_caps = gst_caps_is_equal (caps, othercaps);
   GST_DEBUG_OBJECT (trans, "have_same_caps: %d", trans->have_same_caps);
@@ -858,6 +870,10 @@ failed_configure:
 /* Allocate a buffer using gst_pad_alloc_buffer
  *
  * This function does not do renegotiation on the source pad
+ *
+ * The output buffer is always writable. outbuf can be equal to
+ * inbuf, the caller should be prepared for this and perform 
+ * appropriate refcounting.
  */
 static GstFlowReturn
 gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
@@ -873,6 +889,7 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
    * the old buffer. We will therefore delay the reconfiguration of the
    * element until we have processed this last buffer. */
   trans->delay_configure = TRUE;
+
   /* out_caps is the caps of the src pad gathered through the GST_PAD_CAPS 
      macro. If a set_caps occurs during this function this caps will become
      invalid. We want to keep them during preparation of the output buffer. */
@@ -886,6 +903,13 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
         out_buf);
     if (ret != GST_FLOW_OK)
       goto done;
+
+    /* decrease refcount again if vmethod returned refcounted in_buf. This
+     * is because we need to make sure that the buffer is writable for the
+     * in_place transform. The docs of the vmethod say that you should return
+     * a reffed inbuf, which is exactly what we don't want :), oh well.. */
+    if (in_buf == *out_buf)
+      gst_buffer_unref (in_buf);
   }
 
   /* See if we want to prepare the buffer for in place output */
@@ -893,18 +917,24 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
       && bclass->transform_ip) {
     if (gst_buffer_is_writable (in_buf)) {
       if (trans->have_same_caps) {
-        /* Input buffer is already writable and caps are the same, just ref and return it */
+        /* Input buffer is already writable and caps are the same, return input as
+         * output buffer. We don't take an additional ref since that would make the
+         * output buffer not writable anymore. Caller should be prepared to deal
+         * with proper refcounting of input/output buffers. */
         *out_buf = in_buf;
-        gst_buffer_ref (in_buf);
+        GST_LOG_OBJECT (trans, "reuse input buffer");
       } else {
         /* Writable buffer, but need to change caps => subbuffer */
         *out_buf = gst_buffer_create_sub (in_buf, 0, GST_BUFFER_SIZE (in_buf));
         gst_caps_replace (&GST_BUFFER_CAPS (*out_buf), out_caps);
+        GST_LOG_OBJECT (trans, "created sub-buffer of input buffer");
       }
+      /* we are done now */
       goto done;
     } else {
       /* Make a writable buffer below and copy the data */
       copy_inbuf = TRUE;
+      GST_LOG_OBJECT (trans, "need to copy input buffer to new output buffer");
     }
   }
 
@@ -941,6 +971,7 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
 done:
   if (out_caps)
     gst_caps_unref (out_caps);
+
   trans->delay_configure = FALSE;
 
   return ret;
@@ -1041,20 +1072,26 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
         buf);
   } else {
     /* if we are configured, request a buffer with the src caps */
-    GstCaps *srccaps = gst_pad_get_negotiated_caps (trans->srcpad);
-    GstCaps *sinkcaps = gst_pad_get_negotiated_caps (trans->sinkpad);
+    GstCaps *srccaps;
+    GstCaps *sinkcaps;
+    gboolean configured;
 
-    if (!srccaps)
+    /* take lock, peek if the caps are ok */
+    GST_OBJECT_LOCK (trans->sinkpad);
+    sinkcaps = GST_PAD_CAPS (trans->sinkpad);
+    configured = (sinkcaps == NULL || gst_caps_is_equal (sinkcaps, caps));
+    GST_OBJECT_UNLOCK (trans->sinkpad);
+    if (!configured)
       goto not_configured;
 
-    if (sinkcaps != NULL) {
-      if (sinkcaps != caps || !gst_caps_is_equal (sinkcaps, caps)) {
-        gst_caps_unref (sinkcaps);
-        gst_caps_unref (srccaps);
-        goto not_configured;
-      }
-      gst_caps_unref (sinkcaps);
-    }
+    /* take lock on srcpad to grab the caps, caps can change when pushing a
+     * buffer. */
+    GST_OBJECT_LOCK (trans->srcpad);
+    if ((srccaps = GST_PAD_CAPS (trans->srcpad)))
+      gst_caps_ref (srccaps);
+    GST_OBJECT_UNLOCK (trans->srcpad);
+    if (!srccaps)
+      goto not_configured;
 
     GST_DEBUG_OBJECT (trans, "calling transform_size");
     if (!gst_base_transform_transform_size (trans,
@@ -1066,6 +1103,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     res =
         gst_pad_alloc_buffer_and_set_caps (trans->srcpad, offset, new_size,
         srccaps, buf);
+
     gst_caps_unref (srccaps);
   }
 
@@ -1075,12 +1113,25 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
        the alloc_buffer served to transmit caps information but we can't use the
        buffer. fall through and allocate a buffer corresponding to our sink
        caps, if any */
-    GstCaps *sinkcaps = gst_pad_get_negotiated_caps (trans->sinkpad);
-    GstCaps *srccaps = gst_pad_get_negotiated_caps (trans->srcpad);
+    GstCaps *sinkcaps;
+    GstCaps *srccaps;
 
+    GST_OBJECT_LOCK (trans->sinkpad);
+    if ((sinkcaps = GST_PAD_CAPS (trans->sinkpad)))
+      gst_caps_ref (sinkcaps);
+    GST_OBJECT_UNLOCK (trans->sinkpad);
     if (!sinkcaps)
       goto not_configured;
 
+    GST_OBJECT_LOCK (trans->srcpad);
+    if ((srccaps = GST_PAD_CAPS (trans->srcpad)))
+      gst_caps_ref (srccaps);
+    GST_OBJECT_UNLOCK (trans->srcpad);
+    if (!srccaps) {
+      gst_caps_unref (sinkcaps);
+      goto not_configured;
+    }
+
     if (!gst_base_transform_transform_size (trans,
             GST_PAD_DIRECTION (trans->srcpad), srccaps, GST_BUFFER_SIZE (*buf),
             sinkcaps, &new_size)) {
@@ -1088,16 +1139,17 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
       gst_caps_unref (sinkcaps);
       goto unknown_size;
     }
+    /* don't need the caps anymore now */
+    gst_caps_unref (srccaps);
 
     gst_buffer_unref (*buf);
 
     *buf = gst_buffer_new_and_alloc (new_size);
-    gst_buffer_set_caps (*buf, sinkcaps);
     GST_BUFFER_OFFSET (*buf) = offset;
-    res = GST_FLOW_OK;
+    /* set caps, gives away the ref */
+    GST_BUFFER_CAPS (*buf) = sinkcaps;
 
-    gst_caps_unref (srccaps);
-    gst_caps_unref (sinkcaps);
+    res = GST_FLOW_OK;
   }
 
 done:
@@ -1108,18 +1160,32 @@ done:
 
 not_configured:
   {
-    /* let the default allocator handle it */
+    /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "not configured");
     gst_buffer_replace (buf, NULL);
-    res = GST_FLOW_OK;
+    if (trans->passthrough || trans->always_in_place) {
+      /* ...by calling alloc_buffer without setting caps on the src pad, which
+       * will force negotiation in the chain function. */
+      res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
+    } else {
+      /* ...by letting the default handler create a buffer */
+      res = GST_FLOW_OK;
+    }
     goto done;
   }
 unknown_size:
   {
-    /* let the default allocator handle it */
+    /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "unknown size");
     gst_buffer_replace (buf, NULL);
-    res = GST_FLOW_OK;
+    if (trans->passthrough || trans->always_in_place) {
+      /* ...by calling alloc_buffer without setting caps on the src pad, which
+       * will force negotiation in the chain function. */
+      res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
+    } else {
+      /* ...by letting the default handler create a buffer */
+      res = GST_FLOW_OK;
+    }
     goto done;
   }
 }
@@ -1177,12 +1243,6 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
       gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
           &start, &stop, &time);
 
-      if (trans->segment.format != format)
-        gst_segment_init (&trans->segment, format);
-
-      gst_segment_set_newsegment_full (&trans->segment, update, rate, arate,
-          format, start, stop, time);
-
       trans->have_newsegment = TRUE;
 
       if (format == GST_FORMAT_TIME) {
@@ -1200,6 +1260,9 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
             trans->segment.start, trans->segment.stop,
             trans->segment.time, trans->segment.accum);
       }
+
+      gst_segment_set_newsegment_full (&trans->segment, update, rate, arate,
+          format, start, stop, time);
       break;
     }
     default:
@@ -1382,8 +1445,11 @@ no_qos:
     if (!success)
       goto configure_failed;
   }
+
 skip:
-  gst_buffer_unref (inbuf);
+  /* only unref input buffer if we allocated a new outbuf buffer */
+  if (*outbuf != inbuf)
+    gst_buffer_unref (inbuf);
 
   return ret;
 
@@ -1412,7 +1478,8 @@ no_buffer:
   }
 configure_failed:
   {
-    gst_buffer_unref (inbuf);
+    if (*outbuf != inbuf)
+      gst_buffer_unref (inbuf);
     GST_DEBUG_OBJECT (trans, "could not negotiate");
     return GST_FLOW_NOT_NEGOTIATED;
   }
