@@ -33,12 +33,12 @@
  *   <listitem><para>
  *     Pads are added to the collection with gst_collect_pads_add_pad()/
  *     gst_collect_pads_remove_pad(). The pad
- *     has to be a sinkpad. The chain function of the pad is
+ *     has to be a sinkpad. The chain and event functions of the pad are
  *     overridden. The element_private of the pad is used to store
- *     private information.
+ *     private information for the collectpads.
  *   </para></listitem>
  *   <listitem><para>
- *     For each pad, data is queued in the chain function or by
+ *     For each pad, data is queued in the _chain function or by
  *     performing a pull_range.
  *   </para></listitem>
  *   <listitem><para>
@@ -64,9 +64,12 @@
  *   </para></listitem>
  *   <listitem><para>
  *     gst_collect_pads_collect() and gst_collect_pads_collect_range() can be used by
- *     elements that start a #GstTask to drive the collect_pads.
+ *     elements that start a #GstTask to drive the collect_pads. This feature is however
+ *     not yet implemented.
  *   </para></listitem>
  * </itemizedlist>
+ *
+ * Last reviewed on 2006-05-10 (0.10.6)
  */
 
 #include "gstcollectpads.h"
@@ -74,16 +77,18 @@
 GST_DEBUG_CATEGORY_STATIC (collect_pads_debug);
 #define GST_CAT_DEFAULT collect_pads_debug
 
-GST_BOILERPLATE (GstCollectPads, gst_collect_pads, GstObject, GST_TYPE_OBJECT)
+GST_BOILERPLATE (GstCollectPads, gst_collect_pads, GstObject, GST_TYPE_OBJECT);
 
-     static GstFlowReturn gst_collect_pads_chain (GstPad * pad,
-    GstBuffer * buffer);
-     static gboolean gst_collect_pads_event (GstPad * pad, GstEvent * event);
-     static void gst_collect_pads_finalize (GObject * object);
-     static void gst_collect_pads_init (GstCollectPads * pads,
+static void gst_collect_pads_clear (GstCollectPads * pads,
+    GstCollectData * data);
+static GstFlowReturn gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer);
+static gboolean gst_collect_pads_event (GstPad * pad, GstEvent * event);
+static void gst_collect_pads_finalize (GObject * object);
+static void gst_collect_pads_init (GstCollectPads * pads,
     GstCollectPadsClass * g_class);
 
-     static void gst_collect_pads_base_init (gpointer g_class)
+static void
+gst_collect_pads_base_init (gpointer g_class)
 {
   GST_DEBUG_CATEGORY_INIT (collect_pads_debug, "collectpads", 0,
       "GstCollectPads");
@@ -107,6 +112,11 @@ gst_collect_pads_init (GstCollectPads * pads, GstCollectPadsClass * g_class)
   pads->queuedpads = 0;
   pads->eospads = 0;
   pads->started = FALSE;
+
+  /* members to manage the pad list */
+  pads->abidata.ABI.pad_lock = g_mutex_new ();
+  pads->abidata.ABI.pad_cookie = 0;
+  pads->abidata.ABI.pad_list = NULL;
 }
 
 static void
@@ -115,21 +125,26 @@ gst_collect_pads_finalize (GObject * object)
   GSList *collected;
   GstCollectPads *pads = GST_COLLECT_PADS (object);
 
-  gst_collect_pads_stop (pads);
+  GST_DEBUG ("finalize");
+
   g_cond_free (pads->cond);
+  g_mutex_free (pads->abidata.ABI.pad_lock);
 
   /* Remove pads */
-  for (collected = pads->data; collected; collected = g_slist_next (collected)) {
+  collected = pads->abidata.ABI.pad_list;
+  for (; collected; collected = g_slist_next (collected)) {
     GstCollectData *pdata = (GstCollectData *) collected->data;
 
     if (pdata->pad) {
+      GST_DEBUG ("finalize pad %s:%s", GST_DEBUG_PAD_NAME (pdata->pad));
       gst_object_unref (pdata->pad);
+      pdata->pad = NULL;
     }
+    g_free (pdata);
   }
   /* Free pads list */
   g_slist_free (pads->data);
-
-  /* FIXME, free data */
+  g_slist_free (pads->abidata.ABI.pad_list);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -181,10 +196,12 @@ gst_collect_pads_set_function (GstCollectPads * pads,
  * gst_collect_pads_add_pad:
  * @pads: the collectspads to use
  * @pad: the pad to add
- * @size: the size of the returned GstCollectData structure
+ * @size: the size of the returned #GstCollectData structure
  *
  * Add a pad to the collection of collect pads. The pad has to be
- * a sinkpad.
+ * a sinkpad. The refcount of the pad is incremented. Use
+ * gst_collect_pads_remove_pad() to remove the pad from the collection
+ * again.
  *
  * You specify a size for the returned #GstCollectData structure
  * so that you can use it to store additional information.
@@ -205,22 +222,26 @@ gst_collect_pads_add_pad (GstCollectPads * pads, GstPad * pad, guint size)
   g_return_val_if_fail (GST_PAD_IS_SINK (pad), NULL);
   g_return_val_if_fail (size >= sizeof (GstCollectData), NULL);
 
+  GST_DEBUG ("adding pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+
   data = g_malloc0 (size);
   data->collect = pads;
   data->pad = gst_object_ref (pad);
   data->buffer = NULL;
+  data->pos = 0;
   gst_segment_init (&data->segment, GST_FORMAT_UNDEFINED);
   data->abidata.ABI.flushing = FALSE;
   data->abidata.ABI.new_segment = FALSE;
+  data->abidata.ABI.eos = FALSE;
 
-  GST_OBJECT_LOCK (pads);
-  pads->data = g_slist_append (pads->data, data);
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  pads->abidata.ABI.pad_list =
+      g_slist_append (pads->abidata.ABI.pad_list, data);
   gst_pad_set_chain_function (pad, GST_DEBUG_FUNCPTR (gst_collect_pads_chain));
   gst_pad_set_event_function (pad, GST_DEBUG_FUNCPTR (gst_collect_pads_event));
   gst_pad_set_element_private (pad, data);
-  pads->numpads++;
-  pads->cookie++;
-  GST_OBJECT_UNLOCK (pads);
+  pads->abidata.ABI.pad_cookie++;
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
 
   return data;
 }
@@ -238,15 +259,18 @@ find_pad (GstCollectData * data, GstPad * pad)
  * @pads: the collectspads to use
  * @pad: the pad to remove
  *
- * Remove a pad from the collection of collect pads.
+ * Remove a pad from the collection of collect pads. This function will also
+ * free the #GstCollectData and all the resources that were allocated with 
+ * gst_collect_pads_add_pad().
  *
- * Returns: TRUE if the pad could be removed.
+ * Returns: %TRUE if the pad could be removed.
  *
  * MT safe.
  */
 gboolean
 gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
 {
+  GstCollectData *data;
   GSList *list;
 
   g_return_val_if_fail (pads != NULL, FALSE);
@@ -254,20 +278,58 @@ gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
   g_return_val_if_fail (pad != NULL, FALSE);
   g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
 
-  GST_OBJECT_LOCK (pads);
-  list = g_slist_find_custom (pads->data, pad, (GCompareFunc) find_pad);
-  if (list) {
-    g_free (list->data);
-    pads->data = g_slist_delete_link (pads->data, list);
-    gst_object_unref (pad);
-  }
-  pads->numpads--;
-  /* FIXME : if the pad has data queued we should decrease the number of
-     queuedpads */
-  pads->cookie++;
-  GST_OBJECT_UNLOCK (pads);
+  GST_DEBUG ("removing pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
-  return list != NULL;
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  list =
+      g_slist_find_custom (pads->abidata.ABI.pad_list, pad,
+      (GCompareFunc) find_pad);
+  if (!list)
+    goto unknown_pad;
+
+  data = (GstCollectData *) list->data;
+
+  GST_DEBUG ("found pad %s:%s at %p", GST_DEBUG_PAD_NAME (pad), data);
+
+  /* clear the stuff we configured */
+  gst_pad_set_chain_function (pad, NULL);
+  gst_pad_set_event_function (pad, NULL);
+  /* FIXME, check that freeing the private data does not causes
+   * crashes in the streaming thread */
+  gst_pad_set_element_private (pad, NULL);
+
+  /* backward compat, also remove from data if stopped */
+  if (!pads->started) {
+    GSList *dlist;
+
+    dlist = g_slist_find_custom (pads->data, pad, (GCompareFunc) find_pad);
+    if (dlist) {
+      pads->data = g_slist_delete_link (pads->data, dlist);
+    }
+  }
+  /* remove from the pad list */
+  pads->abidata.ABI.pad_list =
+      g_slist_delete_link (pads->abidata.ABI.pad_list, list);
+  pads->abidata.ABI.pad_cookie++;
+
+  /* clean and free the collect data */
+  gst_object_unref (data->pad);
+  if (data->buffer)
+    gst_buffer_unref (data->buffer);
+  g_free (data);
+
+  /* signal waiters because something changed */
+  GST_COLLECT_PADS_BROADCAST (pads);
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
+
+  return TRUE;
+
+unknown_pad:
+  {
+    GST_WARNING ("cannot remove unknown pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+    GST_COLLECT_PADS_PAD_UNLOCK (pads);
+    return FALSE;
+  }
 }
 
 /**
@@ -277,7 +339,9 @@ gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
  *
  * Check if a pad is active.
  *
- * Returns: TRUE if the pad is active.
+ * This function is currently not implemented.
+ *
+ * Returns: %TRUE if the pad is active.
  *
  * MT safe.
  */
@@ -299,10 +363,11 @@ gst_collect_pads_is_active (GstCollectPads * pads, GstPad * pad)
  * @pads: the collectspads to use
  *
  * Collect data on all pads. This function is usually called
- * from a GstTask function in an element. This function is
- * currently not implemented.
+ * from a #GstTask function in an element. 
  *
- * Returns: GstFlowReturn of the operation.
+ * This function is currently not implemented.
+ *
+ * Returns: #GstFlowReturn of the operation.
  *
  * MT safe.
  */
@@ -314,7 +379,7 @@ gst_collect_pads_collect (GstCollectPads * pads)
 
   g_warning ("gst_collect_pads_collect() is not implemented");
 
-  return GST_FLOW_ERROR;
+  return GST_FLOW_NOT_SUPPORTED;
 }
 
 /**
@@ -324,10 +389,11 @@ gst_collect_pads_collect (GstCollectPads * pads)
  * @length: the length to collect
  *
  * Collect data with @offset and @length on all pads. This function
- * is typically called in the getrange function of an element. This
- * function is currently not implemented.
+ * is typically called in the getrange function of an element. 
  *
- * Returns: GstFlowReturn of the operation.
+ * This function is currently not implemented.
+ *
+ * Returns: #GstFlowReturn of the operation.
  *
  * MT safe.
  */
@@ -340,7 +406,59 @@ gst_collect_pads_collect_range (GstCollectPads * pads, guint64 offset,
 
   g_warning ("gst_collect_pads_collect_range() is not implemented");
 
-  return GST_FLOW_ERROR;
+  return GST_FLOW_NOT_SUPPORTED;
+}
+
+/* FIXME, I think this function is used to work around bad behaviour
+ * of elements that add pads to themselves without activating them.
+ *
+ * Must be called with PAD_LOCK.
+ */
+static void
+gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
+    gboolean flushing)
+{
+  GSList *walk = NULL;
+
+  /* Update the pads flushing flag */
+  for (walk = pads->data; walk; walk = g_slist_next (walk)) {
+    GstCollectData *cdata = walk->data;
+
+    if (GST_IS_PAD (cdata->pad)) {
+      GST_OBJECT_LOCK (cdata->pad);
+      if (flushing)
+        GST_PAD_SET_FLUSHING (cdata->pad);
+      else
+        GST_PAD_UNSET_FLUSHING (cdata->pad);
+      cdata->abidata.ABI.flushing = flushing;
+      gst_collect_pads_clear (pads, cdata);
+      GST_OBJECT_UNLOCK (cdata->pad);
+    }
+  }
+}
+
+/**
+ * gst_collect_pads_set_flushing:
+ * @pads: the collectspads to use
+ * @flushing: desired state of the pads
+ *
+ * Change the flushing state of all the pads in the collection. No pad
+ * is able to accept anymore data when @flushing is %TRUE. Calling this
+ * function with @flushing %TRUE makes @pads accept data again.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.7.
+ */
+void
+gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
+{
+  g_return_if_fail (pads != NULL);
+  g_return_if_fail (GST_IS_COLLECT_PADS (pads));
+
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  gst_collect_pads_set_flushing_unlocked (pads, flushing);
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
 }
 
 /**
@@ -354,29 +472,33 @@ gst_collect_pads_collect_range (GstCollectPads * pads, guint64 offset,
 void
 gst_collect_pads_start (GstCollectPads * pads)
 {
-  GSList *walk = NULL;
+  GSList *collected;
 
   g_return_if_fail (pads != NULL);
   g_return_if_fail (GST_IS_COLLECT_PADS (pads));
 
   GST_DEBUG_OBJECT (pads, "starting collect pads");
 
+  /* make sure stop and collect cannot be called anymore */
   GST_OBJECT_LOCK (pads);
-  /* Set our pads as non flushing */
-  walk = pads->data;
-  while (walk) {
-    GstCollectData *cdata = walk->data;
 
-    if (GST_IS_PAD (cdata->pad)) {
-      GST_OBJECT_LOCK (cdata->pad);
-      GST_PAD_UNSET_FLUSHING (cdata->pad);
-      GST_OBJECT_UNLOCK (cdata->pad);
-    }
+  /* make pads streamable */
+  GST_COLLECT_PADS_PAD_LOCK (pads);
 
-    walk = g_slist_next (walk);
+  /* loop over the master pad list and reset the segment */
+  collected = pads->abidata.ABI.pad_list;
+  for (; collected; collected = g_slist_next (collected)) {
+    GstCollectData *data;
+
+    data = collected->data;
+    gst_segment_init (&data->segment, GST_FORMAT_UNDEFINED);
   }
+
+  gst_collect_pads_set_flushing_unlocked (pads, FALSE);
+
   /* Start collect pads */
   pads->started = TRUE;
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
   GST_OBJECT_UNLOCK (pads);
 }
 
@@ -392,31 +514,44 @@ gst_collect_pads_start (GstCollectPads * pads)
 void
 gst_collect_pads_stop (GstCollectPads * pads)
 {
-  GSList *walk = NULL;
+  GSList *collected;
 
   g_return_if_fail (pads != NULL);
   g_return_if_fail (GST_IS_COLLECT_PADS (pads));
 
   GST_DEBUG_OBJECT (pads, "stopping collect pads");
 
+  /* make sure collect and start cannot be called anymore */
   GST_OBJECT_LOCK (pads);
-  /* Set our pads as flushing */
-  walk = pads->data;
-  while (walk) {
-    GstCollectData *cdata = walk->data;
 
-    if (GST_IS_PAD (cdata->pad)) {
-      GST_OBJECT_LOCK (cdata->pad);
-      GST_PAD_SET_FLUSHING (cdata->pad);
-      GST_OBJECT_UNLOCK (cdata->pad);
-    }
+  /* make pads not accept data anymore */
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  gst_collect_pads_set_flushing_unlocked (pads, TRUE);
 
-    walk = g_slist_next (walk);
-  }
   /* Stop collect pads */
   pads->started = FALSE;
-  /* Wake them up */
+  pads->eospads = 0;
+  pads->queuedpads = 0;
+
+  /* loop over the master pad list and flush buffers */
+  collected = pads->abidata.ABI.pad_list;
+  for (; collected; collected = g_slist_next (collected)) {
+    GstCollectData *data;
+    GstBuffer **buffer_p;
+
+    data = collected->data;
+    if (data->buffer) {
+      buffer_p = &data->buffer;
+      gst_buffer_replace (buffer_p, NULL);
+      data->pos = 0;
+    }
+    data->abidata.ABI.eos = FALSE;
+  }
+
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
+  /* Wake them up so then can end the chain functions. */
   GST_COLLECT_PADS_BROADCAST (pads);
+
   GST_OBJECT_UNLOCK (pads);
 }
 
@@ -443,9 +578,7 @@ gst_collect_pads_peek (GstCollectPads * pads, GstCollectData * data)
   g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), NULL);
   g_return_val_if_fail (data != NULL, NULL);
 
-  result = data->buffer;
-
-  if (result)
+  if ((result = data->buffer))
     gst_buffer_ref (result);
 
   GST_DEBUG ("Peeking at pad %s:%s: buffer=%p",
@@ -472,17 +605,15 @@ GstBuffer *
 gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
 {
   GstBuffer *result;
-  GstBuffer **buffer_p;
 
   g_return_val_if_fail (pads != NULL, NULL);
   g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), NULL);
   g_return_val_if_fail (data != NULL, NULL);
 
-  result = data->buffer;
-  if (result) {
-    buffer_p = &data->buffer;
-    gst_buffer_replace (buffer_p, NULL);
+  if ((result = data->buffer)) {
+    data->buffer = NULL;
     data->pos = 0;
+    /* one less pad with queued data now */
     pads->queuedpads--;
   }
 
@@ -494,6 +625,17 @@ gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
   return result;
 }
 
+/* pop and unref the currently queued buffer, should e called with the LOCK
+ * helt. */
+static void
+gst_collect_pads_clear (GstCollectPads * pads, GstCollectData * data)
+{
+  GstBuffer *buf;
+
+  if ((buf = gst_collect_pads_pop (pads, data)))
+    gst_buffer_unref (buf);
+}
+
 /**
  * gst_collect_pads_available:
  * @pads: the collectspads to query
@@ -510,6 +652,7 @@ gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
  *
  * MT safe.
  */
+/* FIXME, we can do this in the _chain functions */
 guint
 gst_collect_pads_available (GstCollectPads * pads)
 {
@@ -521,18 +664,35 @@ gst_collect_pads_available (GstCollectPads * pads)
 
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
     GstCollectData *pdata;
+    GstBuffer *buffer;
     gint size;
 
     pdata = (GstCollectData *) collected->data;
 
-    if (pdata->buffer == NULL)
+    /* ignore pad with EOS */
+    if (G_UNLIKELY (pdata->abidata.ABI.eos)) {
+      GST_DEBUG ("pad %p is EOS", pdata);
+      continue;
+    }
+
+    /* an empty buffer without EOS is weird when we get here.. */
+    if (G_UNLIKELY ((buffer = pdata->buffer) == NULL)) {
+      GST_WARNING ("pad %p has no buffer", pdata);
       goto not_filled;
+    }
 
-    size = GST_BUFFER_SIZE (pdata->buffer) - pdata->pos;
+    /* this is the size left of the buffer */
+    size = GST_BUFFER_SIZE (buffer) - pdata->pos;
+    GST_DEBUG ("pad %p has %d bytes left", pdata, size);
 
+    /* need to return the min of all available data */
     if (size < result)
       result = size;
   }
+  /* nothing changed, all must be EOS then, return 0 */
+  if (G_UNLIKELY (result == G_MAXUINT))
+    result = 0;
+
   return result;
 
 not_filled:
@@ -565,15 +725,20 @@ gst_collect_pads_read (GstCollectPads * pads, GstCollectData * data,
     guint8 ** bytes, guint size)
 {
   guint readsize;
+  GstBuffer *buffer;
 
   g_return_val_if_fail (pads != NULL, 0);
   g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), 0);
   g_return_val_if_fail (data != NULL, 0);
   g_return_val_if_fail (bytes != NULL, 0);
 
-  readsize = MIN (size, GST_BUFFER_SIZE (data->buffer) - data->pos);
+  /* no buffer, must be EOS */
+  if ((buffer = data->buffer) == NULL)
+    return 0;
 
-  *bytes = GST_BUFFER_DATA (data->buffer) + data->pos;
+  readsize = MIN (size, GST_BUFFER_SIZE (buffer) - data->pos);
+
+  *bytes = GST_BUFFER_DATA (buffer) + data->pos;
 
   return readsize;
 }
@@ -599,80 +764,140 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
     guint size)
 {
   guint flushsize;
+  GstBuffer *buffer;
 
   g_return_val_if_fail (pads != NULL, 0);
   g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), 0);
   g_return_val_if_fail (data != NULL, 0);
 
-  flushsize = MIN (size, GST_BUFFER_SIZE (data->buffer) - data->pos);
+  /* no buffer, must be EOS */
+  if ((buffer = data->buffer) == NULL)
+    return 0;
+
+  /* this is what we can flush at max */
+  flushsize = MIN (size, GST_BUFFER_SIZE (buffer) - data->pos);
 
   data->pos += size;
 
-  if (data->pos >= GST_BUFFER_SIZE (data->buffer)) {
-    GstBuffer *buf;
-
-    buf = gst_collect_pads_pop (pads, data);
-    gst_buffer_unref (buf);
-  }
+  if (data->pos >= GST_BUFFER_SIZE (buffer))
+    /* _clear will also reset data->pos to 0 */
+    gst_collect_pads_clear (pads, data);
 
   return flushsize;
 }
 
-static gboolean
-gst_collect_pads_is_collected (GstCollectPads * pads, GstFlowReturn * ret)
+/* see if pads were added or removed and update our stats. Any pad
+ * added after releasing the PAD_LOCK will get collected in the next
+ * round.
+ *
+ * We can do a quick check by checking the cookies, that get changed
+ * whenever the pad list is updated.
+ *
+ * Must be called with LOCK.
+ */
+static void
+gst_collect_pads_check_pads (GstCollectPads * pads)
+{
+  /* the master list and cookie are protected with the PAD_LOCK */
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  if (G_UNLIKELY (pads->abidata.ABI.pad_cookie != pads->cookie)) {
+    GSList *collected;
+
+    /* clear list and stats */
+    g_slist_free (pads->data);
+    pads->data = NULL;
+    pads->numpads = 0;
+    pads->queuedpads = 0;
+    pads->eospads = 0;
+
+    /* loop over the master pad list */
+    collected = pads->abidata.ABI.pad_list;
+    for (; collected; collected = g_slist_next (collected)) {
+      GstCollectData *data;
+
+      /* update the stats */
+      pads->numpads++;
+      data = collected->data;
+      if (data->buffer)
+        pads->queuedpads++;
+      if (data->abidata.ABI.eos)
+        pads->eospads++;
+
+      /* add to the list of pads to collect */
+      pads->data = g_slist_prepend (pads->data, data);
+    }
+    /* and update the cookie */
+    pads->cookie = pads->abidata.ABI.pad_cookie;
+  }
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
+}
+
+/* checks if all the pads are collected and call the collectfunction
+ *
+ * Should be called with LOCK.
+ *
+ * Returns: The #GstFlowReturn of collection.
+ */
+static GstFlowReturn
+gst_collect_pads_check_collected (GstCollectPads * pads)
 {
   GstFlowReturn flow_ret = GST_FLOW_OK;
-  gboolean res = FALSE;
 
-  g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), FALSE);
+  g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), GST_FLOW_ERROR);
+  g_return_val_if_fail (pads->func != NULL, GST_FLOW_NOT_SUPPORTED);
 
-  /* If all our pads are EOS just collect once */
-  if (pads->eospads == pads->numpads) {
+  /* check for new pads, update stats etc.. */
+  gst_collect_pads_check_pads (pads);
+
+  if (G_UNLIKELY (pads->eospads == pads->numpads)) {
+    /* If all our pads are EOS just collect once to let the element
+     * do its final EOS handling. */
     GST_DEBUG ("All active pads (%d) are EOS, calling %s",
         pads->numpads, GST_DEBUG_FUNCPTR_NAME (pads->func));
     flow_ret = pads->func (pads, pads->user_data);
-    res = TRUE;
-    goto beach;
+  } else {
+    gboolean collected = FALSE;
+
+    /* We call the collected function as long as our condition matches.
+     * FIXME: should we error out if the collect function did not pop anything ?
+     * we can get a busy loop here if the element does not pop from the collect
+     * function
+     */
+    while (((pads->queuedpads + pads->eospads) >= pads->numpads)) {
+      GST_DEBUG ("All active pads (%d + %d >= %d) have data, calling %s",
+          pads->queuedpads, pads->eospads, pads->numpads,
+          GST_DEBUG_FUNCPTR_NAME (pads->func));
+      flow_ret = pads->func (pads, pads->user_data);
+      collected = TRUE;
+
+      /* break on error */
+      if (flow_ret != GST_FLOW_OK)
+        break;
+      /* Don't keep looping after telling the element EOS or flushing */
+      if (pads->queuedpads == 0)
+        break;
+    }
+    if (!collected)
+      GST_DEBUG ("Not all active pads (%d) have data, continuing",
+          pads->numpads);
   }
-
-  /* We call the collected function as long as our condition matches.
-     FIXME: should we error out if the collect function did not pop anything ?
-     we can get a busy loop here if the element does not pop from the collect
-     function */
-  while (((pads->queuedpads + pads->eospads) >= pads->numpads) && pads->func) {
-    GST_DEBUG ("All active pads (%d) have data, calling %s",
-        pads->numpads, GST_DEBUG_FUNCPTR_NAME (pads->func));
-    flow_ret = pads->func (pads, pads->user_data);
-    res = TRUE;
-
-    /* Don't keep looping after telling the element EOS or flushing */
-    if (pads->queuedpads == 0)
-      break;
-  }
-
-beach:
-  if (!res) {
-    GST_DEBUG ("Not all active pads have data, continuing");
-  }
-
-  if (ret) {
-    *ret = flow_ret;
-  }
-
-  return res;
+  return flow_ret;
 }
 
 static gboolean
 gst_collect_pads_event (GstPad * pad, GstEvent * event)
 {
+  gboolean res;
   GstCollectData *data;
   GstCollectPads *pads;
 
   /* some magic to get the managing collect_pads */
   data = (GstCollectData *) gst_pad_get_element_private (pad);
-  if (data == NULL)
+  if (G_UNLIKELY (data == NULL))
     goto not_ours;
 
+  res = TRUE;
+
   pads = data->collect;
 
   GST_DEBUG ("Got %s event on pad %s:%s", GST_EVENT_TYPE_NAME (event),
@@ -681,88 +906,118 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
     {
-      /* forward event to unblock is_collected */
+      /* forward event to unblock check_collected */
       gst_pad_event_default (pad, event);
 
-      /* now unblock the chain function
-         no cond per pad, so they all unblock, non-flushing block again */
+      /* now unblock the chain function.
+       * no cond per pad, so they all unblock, 
+       * non-flushing block again */
       GST_OBJECT_LOCK (pads);
       data->abidata.ABI.flushing = TRUE;
-      GST_COLLECT_PADS_BROADCAST (pads);
+      gst_collect_pads_clear (pads, data);
       GST_OBJECT_UNLOCK (pads);
 
       /* event already cleaned up by forwarding */
-      return TRUE;
+      goto done;
     }
     case GST_EVENT_FLUSH_STOP:
     {
       /* flush the 1 buffer queue */
       GST_OBJECT_LOCK (pads);
       data->abidata.ABI.flushing = FALSE;
-      gst_collect_pads_pop (pads, data);
+      gst_collect_pads_clear (pads, data);
+      /* we need new segment info after the flush */
+      gst_segment_init (&data->segment, GST_FORMAT_UNDEFINED);
+      data->abidata.ABI.new_segment = FALSE;
+      /* if the pad was EOS, remove the EOS flag and
+       * decrement the number of eospads */
+      if (G_UNLIKELY (data->abidata.ABI.eos == TRUE)) {
+        pads->eospads--;
+        data->abidata.ABI.eos = FALSE;
+      }
       GST_OBJECT_UNLOCK (pads);
-      goto beach;
+
+      /* forward event */
+      goto forward;
     }
     case GST_EVENT_EOS:
     {
       GST_OBJECT_LOCK (pads);
-
-      pads->eospads++;
-
-      gst_collect_pads_is_collected (pads, NULL);
-
+      /* if the pad was not EOS, make it EOS and so we
+       * have one more eospad */
+      if (G_LIKELY (data->abidata.ABI.eos == FALSE)) {
+        data->abidata.ABI.eos = TRUE;
+        pads->eospads++;
+      }
+      /* check if we need collecting anything, we ignore the
+       * result. */
+      gst_collect_pads_check_collected (pads);
       GST_OBJECT_UNLOCK (pads);
 
-      /* We eat this event */
+      /* We eat this event, element should do something
+       * in the collected callback. */
       gst_event_unref (event);
-      return TRUE;
+      goto done;
     }
     case GST_EVENT_NEWSEGMENT:
     {
       gint64 start, stop, time;
-      gdouble rate;
+      gdouble rate, arate;
       GstFormat format;
       gboolean update;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format,
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
           &start, &stop, &time);
 
       GST_DEBUG_OBJECT (data->pad, "got newsegment, start %" GST_TIME_FORMAT
           ", stop %" GST_TIME_FORMAT, GST_TIME_ARGS (start),
           GST_TIME_ARGS (stop));
 
-      if (data->segment.format != format)
-        gst_segment_init (&data->segment, format);
-
-      gst_segment_set_newsegment (&data->segment, update, rate, format,
-          start, stop, time);
+      gst_segment_set_newsegment_full (&data->segment, update, rate, arate,
+          format, start, stop, time);
 
       data->abidata.ABI.new_segment = TRUE;
 
-      /* We eat this event */
+      /* we must not forward this event since multiple segments will be 
+       * accumulated and this is certainly not what we want. */
       gst_event_unref (event);
-      return TRUE;
+      /* FIXME: collect-pads based elements need to create their own newsegment
+       * event (and only one really)
+       * (a) make the segment part of the GstCollectData structure of each pad,
+       * so you can just check that once you have a buffer queued on that pad,
+       * (b) you can override a pad's event function with your own,
+       * catch the newsegment event and then pass it on to the original
+       * gstcollectpads event function
+       * (that's what avimux does for something IIRC)
+       * see #340060
+       */
+      goto done;
     }
     default:
-      goto beach;
+      /* forward other events */
+      goto forward;
   }
 
-beach:
-  return gst_pad_event_default (pad, event);
+forward:
+  res = gst_pad_event_default (pad, event);
+
+done:
+  return res;
 
   /* ERRORS */
 not_ours:
   {
-    GST_DEBUG ("collect_pads not ours");
+    GST_WARNING ("not our pad");
     return FALSE;
   }
 }
 
 /* For each buffer we receive we check if our collected condition is reached
-   and if so we call the collected function. When this is done we check if
-   data has been unqueued. If data is still queued we wait holding the stream
-   lock to make sure no EOS event can happen while we are ready to be
-   collected */
+ * and if so we call the collected function. When this is done we check if
+ * data has been unqueued. If data is still queued we wait holding the stream
+ * lock to make sure no EOS event can happen while we are ready to be
+ * collected 
+ */
 static GstFlowReturn
 gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -776,7 +1031,7 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
 
   /* some magic to get the managing collect_pads */
   data = (GstCollectData *) gst_pad_get_element_private (pad);
-  if (data == NULL)
+  if (G_UNLIKELY (data == NULL))
     goto not_ours;
 
   pads = data->collect;
@@ -785,11 +1040,14 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   GST_OBJECT_LOCK (pads);
 
   /* if not started, bail out */
-  if (!pads->started)
+  if (G_UNLIKELY (!pads->started))
     goto not_started;
   /* check if this pad is flushing */
-  if (data->abidata.ABI.flushing)
+  if (G_UNLIKELY (data->abidata.ABI.flushing))
     goto flushing;
+  /* pad was EOS, we can refuse this data */
+  if (G_UNLIKELY (data->abidata.ABI.eos))
+    goto unexpected;
 
   GST_DEBUG ("Queuing buffer %p for pad %s:%s", buffer,
       GST_DEBUG_PAD_NAME (pad));
@@ -799,48 +1057,92 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   buffer_p = &data->buffer;
   gst_buffer_replace (buffer_p, buffer);
 
-  if (data->segment.format == GST_FORMAT_TIME
-      && GST_BUFFER_TIMESTAMP_IS_VALID (buffer))
-    gst_segment_set_last_stop (&data->segment, GST_FORMAT_TIME,
-        GST_BUFFER_TIMESTAMP (buffer));
+  /* update segment last position if in TIME */
+  if (G_LIKELY (data->segment.format == GST_FORMAT_TIME)) {
+    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
-  /* Check if our collected condition is matched and call the collected function
-     if it is */
-  gst_collect_pads_is_collected (pads, &ret);
-
-  /* We still have data queued on this pad, wait for something to happen */
-  while (data->buffer != NULL) {
-    GST_DEBUG ("Pad %s:%s has a buffer queued, waiting",
-        GST_DEBUG_PAD_NAME (pad));
-    GST_COLLECT_PADS_WAIT (pads);
-    GST_DEBUG ("Pad %s:%s resuming", GST_DEBUG_PAD_NAME (pad));
-    /* after a signal,  we could be stopped */
-    if (!pads->started)
-      goto not_started;
-    if (data->abidata.ABI.flushing)
-      goto flushing;
+    if (GST_CLOCK_TIME_IS_VALID (timestamp))
+      gst_segment_set_last_stop (&data->segment, GST_FORMAT_TIME, timestamp);
   }
 
+  /* While we have data queued on this pad try to collect stuff */
+  do {
+    /* Check if our collected condition is matched and call the collected function
+     * if it is */
+    ret = gst_collect_pads_check_collected (pads);
+    /* when an error occurs, we want to report this back to the caller ASAP
+     * without having to block if the buffer was not popped */
+    if (G_UNLIKELY (ret != GST_FLOW_OK))
+      goto error;
+
+    /* data was consumed, we can exit and accept new data */
+    if (data->buffer == NULL)
+      break;
+
+    GST_DEBUG ("Pad %s:%s has a buffer queued, waiting",
+        GST_DEBUG_PAD_NAME (pad));
+
+    /* wait to be collected, this must happen from another thread triggered
+     * by the _chain function of another pad. We release the lock so we
+     * can get stopped or flushed as well. We can however not get EOS
+     * because we still hold the STREAM_LOCK. */
+    GST_COLLECT_PADS_WAIT (pads);
+
+    GST_DEBUG ("Pad %s:%s resuming", GST_DEBUG_PAD_NAME (pad));
+
+    /* after a signal, we could be stopped */
+    if (G_UNLIKELY (!pads->started))
+      goto not_started;
+    /* check if this pad is flushing */
+    if (G_UNLIKELY (data->abidata.ABI.flushing))
+      goto flushing;
+  }
+  while (data->buffer != NULL);
+
+unlock_done:
   GST_OBJECT_UNLOCK (pads);
 
+done:
+  gst_buffer_unref (buffer);
+
   return ret;
 
   /* ERRORS */
 not_ours:
   {
-    GST_DEBUG ("collect_pads not ours");
-    return GST_FLOW_ERROR;
+    /* pretty fatal this one, can't post an error though... */
+    GST_WARNING ("not our pad");
+    ret = GST_FLOW_ERROR;
+    goto done;
   }
 not_started:
   {
-    GST_OBJECT_UNLOCK (pads);
-    GST_DEBUG ("collect_pads not started");
-    return GST_FLOW_WRONG_STATE;
+    GST_DEBUG ("not started");
+    gst_collect_pads_clear (pads, data);
+    ret = GST_FLOW_WRONG_STATE;
+    goto unlock_done;
   }
 flushing:
   {
-    GST_OBJECT_UNLOCK (pads);
-    GST_DEBUG ("collect_pads %s:%s is flushing", GST_DEBUG_PAD_NAME (pad));
-    return GST_FLOW_WRONG_STATE;
+    GST_DEBUG ("pad %s:%s is flushing", GST_DEBUG_PAD_NAME (pad));
+    gst_collect_pads_clear (pads, data);
+    ret = GST_FLOW_WRONG_STATE;
+    goto unlock_done;
+  }
+unexpected:
+  {
+    /* we should not post an error for this, just inform upstream that
+     * we don't expect anything anymore */
+    GST_DEBUG ("pad %s:%s is eos", GST_DEBUG_PAD_NAME (pad));
+    ret = GST_FLOW_UNEXPECTED;
+    goto unlock_done;
+  }
+error:
+  {
+    /* we print the error, the element should post a reasonable error
+     * message for fatal errors */
+    GST_DEBUG ("collect failed, reason %d (%s)", ret, gst_flow_get_name (ret));
+    gst_collect_pads_clear (pads, data);
+    goto unlock_done;
   }
 }
